/* -----------------------------------------------------------------------------
*
*  npmysql.c
*
*  ANTz - realtime 3D data visualization tools for the real-world, based on NPE.
*
*  ANTz is hosted at http://openantz.com and NPE at http://neuralphysics.org
*
*  Written in 2010-2012 by Shane Saxon - contact@saxondigital.net
*
*  Please see main.c for a complete list of additional code contributors.
*
*  To the extent possible under law, the author(s) have dedicated all copyright 
*  and related and neighboring rights to this software to the public domain
*  worldwide. This software is distributed without any warranty.
*
*  Released under the CC0 license, which is GPL compatible.
*
*  You should have received a copy of the CC0 Public Domain Dedication along
*  with this software (license file named COPYING.txt). If not, see
*  http://creativecommons.org/publicdomain/zero/1.0/
*
* --------------------------------------------------------------------------- */

#include "npmysql.h"

enum
{
	CHANNELS = 0,
	CHMAP = 1,
	NODES = 2,
	TAGS = 3
};

enum {
	WHERE = 0,
	ORDERBY = 1,
};





enum
{
	dbInsert = 0,
	dbCreate = 1
};

enum {
	RESULTS = 0,
	RESULT = 1
};

enum {
	MYSQL_RESULT = 0
};


int npFreeDbObject(pdbConnectMysql connect, pdatabaseObject object, void* dataRef)
{
	printf("\n 33 33 33 npFreeDbObject\n");
	switch(object->type)
	{
		case MySQL:
			npFreeMysqlObject(connect, object->dbObject, dataRef);
			break;
	}

	printf("\n44 44 44 Freeing npDbObject\n");
	free(object);

	return 0;
}


void npAddRecordTag (pNPrecordTag recordTag, void* dataRef)
{
	int i = 0;
	pNPrecordTag searchTag = NULL;
	pData data = (pData) dataRef;
	
	//if identical recordID && tableID then replace existing
	for (i=0; i < kNPrecordTagListMax; i++)
	{
		searchTag = data->io.gl.hud.tags.recordList[i];
		
		//reached end of list add record
		if (searchTag == NULL)
		{	
			if (data->io.gl.hud.tags.recordCount >= kNPrecordTagListMax - 1)
			{
				printf ("error 4834 - kNPrecordTagListMax exceeded\n");
				return;
			}
			//add the tag to the end of the list
			data->io.gl.hud.tags.recordList[i] = recordTag;
			data->io.gl.hud.tags.recordCount++;
			
			printf("record_id: %d  tag imported\n", recordTag->recordID);
			i = kNPrecordTagListMax; //exit loop
			
		}		//if identical record then update it
		else if ( searchTag->recordID == recordTag->recordID
				 && searchTag->tableID == recordTag->tableID )
		{
			data->io.gl.hud.tags.recordList[i] = recordTag;
			
			free(searchTag);	//debug make sure this okay, debug zz
			
			printf("record_id: %d  tag updated\n", recordTag->recordID);
			i = kNPrecordTagListMax; //exit loop
		}
	}
}


int npMapTraverseTreeLL(char* buffer, int* index, pNPnode node, int format, void* dataRef)
{
	pData data = (pData) dataRef;
	int i = 0;
	int count = 0;
	
	//	printf("\nIndex is %d :: node->branchLevel : %d :: node->childCount : %d", *index, node->branchLevel, node->childCount);
	if (node->branchLevel > 0)
	{
		count = npWriteNode(buffer, node, format, dataRef);
		buffer[count] = '\0';
		//	printf("\nIndex : %d", *index);
		//	printf("\nchild : %s", buffer[*index]);
		//getch();
		(*index)++;
		data->io.write[kNPnode] = buffer;
	}
	
	//recursively calls this function for all children
	for (i=0; i < node->childCount; i++)//node->childCount; j++)
	{
		if (node->child[i] != NULL
			&& node->child[i]->child[0] != node)	//don't write twice, this is link B so skip
		{
			count += npMapTraverseTreeLL(buffer, index, node->child[i], format, dataRef);
		}
	}
	//	printf("npMapTraverseTreeLL totalCount :: %d", count);
	
	
	//return count;
	return count;
}


pNPdatabases npdbGetList(void* dataRef)
{
//	char **dbList;
	pdatabaseObject databaseResult = NULL;
	struct mysqlResult* mysqlResult = NULL;
	pNPdatabases dbList = NULL;
	int i = 0;
	int x = 0;
	MYSQL_ROW row;
	//MYSQL_RES *mysqlResult;
	int numFields;
	int numRows;
	unsigned long* fieldLengths;
	pdbNewConnect connection;
	pData data = (pData) dataRef;
	connection = data->io.connect;
	
	databaseResult = npdbCtrl(connection, DATABASE, NULL, NULL, NULL, NULL, NULL, NULL, Show, dataRef);
	mysqlResult = ((struct dbResult*)databaseResult->dbObject)->resultObj;
	
	numFields = mysql_num_fields(mysqlResult->result);
	numRows = mysql_num_rows(mysqlResult->result);
	
	dbList = malloc(sizeof(pNPdatabases));
	dbList->list = malloc(sizeof(char*) * numRows);
	printf("\n");
	
	// Fix this for general cases, debug db
	while ((row = mysql_fetch_row(mysqlResult->result)))
	{
		fieldLengths = mysql_fetch_lengths(mysqlResult->result);

		for(i = 0; i < numFields; i++)
		{
			printf("\nField Length : %lu\n", fieldLengths[i]);
	//		dbList[x] = malloc(sizeof(char) * fieldLengths[i]);
			dbList->list[x] = malloc((sizeof(char) * (fieldLengths[i]))+1);

			printf("%s ", row[i] ? row[i] : "NULL");
			row[i][fieldLengths[i]] = '\0';
			dbList->list[x] = row[i];
			//strcpy(dbList->list[x], row[i]);


			dbList->size = x;
		}
		x++;
		printf("\n");
		
	}
	
	
	
	return dbList;
}


pdatabaseObject npInitDbResults(pdbNewConnect connect)
{
	pdatabaseObject dbResults = NULL;
	
	dbResults = malloc(sizeof(struct databaseObject*));
	dbResults->type = RESULTS;
	dbResults->dbObject = malloc(sizeof(struct dbResults*));
	((struct dbResults*)dbResults->dbObject)->numOfDbResults = 0;
	
	return dbResults; 
}

// Used to be struct dbConnectMysql
void npdbStoreResultMysql(pdbConnectMysql connect, pmysqlQuery myQuery, pmysqlResult myResult, void* dataRef)
{
//	MYSQL_RES *result;
//	myResult->result
	myResult->result = malloc(sizeof(struct mysqlResult*));
//	myResult->result->result;
	((struct mysqlResult*)myResult->result)->result = mysql_store_result(connect->conn);
	
	if(((struct mysqlResult*)myResult->result)->result == NULL)
	{
		printf("\nResult is NULL");
		//pause();
	}
	else {
		printf("\nResult is not NULL");
		//pause();
	}

	
	return;
}

pdatabaseObject npdbStoreResult(pdbNewConnect connect, pdatabaseObject dbQuery, void* dataRef)
{
	//dbResult->dbObject = malloc(sizeof(struct dbResult*)); // Current, debug db
	pdatabaseObject dbResult = npInitDbResult(connect, dataRef);
	
	switch(connect->dbType)
	{
		case MySQL:
			 npdbStoreResultMysql(connect->dbConnection, dbQuery->dbObject, dbResult->dbObject, dataRef);
			break;
		case Oracle:
			break;
	}
	
	return dbResult;
}

pdatabaseObject npdbStoreResultsMySQL(pdbConnectMysql connect, pdatabaseObject dbQueries, pdatabaseObject dbResults, void* dataRef)
{

	return dbResults;
}

// This processes a dbQueries structure, debug db
pdatabaseObject npdbStoreResults(pdbNewConnect connect, pdatabaseObject dbQueries, pdatabaseObject dbResults, void* dataRef)
{

	switch(connect->dbType)
	{
		case MySQL:
			dbResults = npdbStoreResultsMySQL(connect->dbConnection, dbQueries, dbResults, dataRef);
			break;
		case Oracle:
			break;
	}
	
	return dbResults;
}


// Returns 0 if successful
int npFreeMysqlObject(pdbConnectMysql connect, pmysqlObject object, void* dataRef)
{
	// These switch statements should only be for freeing sub objects within mysqlObject
	switch(object->type)
	{
	case TABLE:
		npFreeMysqlTable(connect, object, dataRef);
		break;
	case DATABASE:
		npFreeMysqlDatabase(connect, object, dataRef);
		break;
	case QUERY:
		npFreeMysqlQuery(connect, object, dataRef);
		break;
	}
	// now free mysqlObject

	return 0;
}


int npFreeMysqlTable(pdbConnectMysql connect, pmysqlObject object, void* dataRef)
{
//	int i = 0;
	struct mysqlTable *tmpTable;
	tmpTable = ((struct mysqlTable*)object->myobject);

	printf("\nfree table : %s", tmpTable->name);
//	free(tmpTable->name);
//	free(tmpTable->fields);

//	if(tmpTable->formattedValues != NULL)
//		free(tmpTable->formattedValues);

	free(object);

	return 0;
}

int npFreeMysqlDatabase(pdbConnectMysql connect,pmysqlObject object, void* dataRef)
{
	struct mysqlDatabase *tmpDatabase;
	tmpDatabase = ((struct mysqlDatabase*)object->myobject);
	
	free(tmpDatabase->name);

	free(tmpDatabase);
	free(object);
	return 0;
}

int npFreeChunk(struct chunkObj chunk)
{
	printf("\nChunk size to free: %d", chunk.chunkSize);
	getch();
}

int npFreeChunks(struct chunksObj *chunks)
{
	int i = 0;
	printf("\nNumber of chunks to free: %d", chunks->numOfChunks);
	getch();
	for(i = 0; i < chunks->numOfChunks; i++)
	{
		printf("\nFreeing Chunk #%d", i);
		npFreeChunk(chunks->chunk[i]);
	}

	return 0;
}

int npFreeMysqlQuery(pdbConnectMysql connect, pmysqlObject object, void* dataRef)
{
	int i = 0;
	int x = 0;
	pmysqlQuery tmpQuery;
	tmpQuery = ((struct mysqlQuery*)object->myobject);
//	free(tmpQuery->subject);

//	for(i = 0; i < tmpQuery->numOfValues; i++) // changed type, debug db
//		free(tmpQuery->values[i]);
	printf("\nnpFreeChunks : %p", tmpQuery->values);
//	getch();
	// Clear all query strings, debug db

	if(tmpQuery->values != NULL)
	{
		printf("\nExecute npFreeChunks");
		npFreeChunks(tmpQuery->values);
//		getch();
	}
	else
	{
		printf("\nNo value chunks were allocated");
	//	getch();
	}
	/*
	if(tmpQuery->values != NULL)
		free(tmpQuery->values);
*/
	printf("numOfResults B : %d", tmpQuery->numOfResults);
	for(i = 0; i < tmpQuery->numOfResults; i++)
	{
		printf("\nI:%d", i);
		if(tmpQuery->result[i] != NULL)
		{
			printf("\nNOT NULL");
			//mysql_free_result(tmpQuery->result[i]); // debug db // Sometimes mysql_query return null even on success
		}
	}

//		free(tmpQuery->result[0]);

	for(i = 0; i < 400; i++)
	{
		for(x = 0; x < 67000; x++)
		{
			tmpQuery->query[i][x] = '\0';
		}
	}

	printf("Freeing tmpObject");
	free(tmpQuery);
	printf("Freeing object");
	free(object);
	return 0;
}

int npCreateDatabase(pdbNewConnect connect, char *name, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		npCreateDatabaseMysql(connect->dbConnection, name, dataRef);
		break;
	case Oracle:
		break;
	}

	return 0;
}

pdatabaseObject npdbUseDatabase(pdbNewConnect connect, char *name, void* dataRef)
{
	pdatabaseObject dbQuery = npInitDbQuery(connect);
	pdatabaseObject dbDatabase = npInitDbDatabase(connect, name, dataRef);
	pdatabaseObject dbResult = npInitDbResult(connect, dataRef);
	
	npSetDbQueryObject(connect, dbQuery, dbDatabase, dataRef);
	npSetDbQueryCommand(connect, dbQuery, Use, dataRef); 
	npdbGenQuery(connect, dbQuery, dataRef);
	npdbExecuteQuery(connect, dbQuery, dataRef);
	dbResult = npdbStoreResult(connect, dbQuery, dataRef);
	
	return dbResult;
}

int npUseDatabase(pdbNewConnect connect, char *name, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		npUseDatabaseMysql(connect->dbConnection, name, dataRef);
		break;
	case Oracle:
		break;
	}
	
	return 0;
}


int npCreateTable(pdbNewConnect connect, char *name, char *fields, void* dataRef)
{
	printf("\nnpCreateTable : %d", connect->dbType);

	switch(connect->dbType)
	{
	case MySQL:
		npCreateTableMysql(connect->dbConnection, name, fields, dataRef);
		break;
	case Oracle:
		break;
//	default:
//		npCreateTableMysql(connect->dbConnection, name, fields, dataRef);
//		break;
	}

	return 0;
}

// This should have more parameters for engines and whatnot, debug ll
int npCreateAntzDatabaseMysql(pdbConnectMysql connect, void *dataRef)
{
	npCreateDatabaseMysql(connect, "Antz", dataRef);
	return 0;
}

int npUseDatabaseMysql(pdbConnectMysql connect, char *name, void* dataRef)
{
	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject database = npInitMySQLDatabase(name);

	npSetMysqlQueryObject(connect, query, database, dataRef);
	if(npSetMysqlQueryCommand(connect, query, Use, dataRef) == 0)
	{
		npGenMysqlQuery(query, dataRef);
	//	npNewExecuteMysqlQuery(query, connect);
		npNewExecuteMysqlQuery(connect, query, dataRef);
	//	connect->db = name;
		connect->db = name;
	}

	
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, database, dataRef);
	return 0;
}

int npUseAntzDatabase(pdbNewConnect connect, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		npUseAntzDatabaseMysql(connect->dbConnection, dataRef);
		break;
	case Oracle:
		break;
	}

	return 0;
}

int	npUseAntzDatabaseMysql(pdbConnectMysql connect, void* dataRef)
{
//	pData data = (pData) npGetDataRef();
	npUseDatabaseMysql(connect, "Antz", dataRef);
//	npUseDatabaseMysql(connect, "Antz");

	return 0;
}

pdatabaseObject npInitDbDatabase(pdbNewConnect connect, char *name, void* dataRef)
{
	pdatabaseObject objectInstance = malloc(sizeof(pdatabaseObject));
	void* object = NULL;

	switch(connect->dbType)
	{
		case MySQL:
			object = npInitMySQLDatabase(name);
			break;
		case Oracle:
			break;
	}
	objectInstance->dbObject = object;
	return objectInstance;
}



pdatabaseObject npdbCreateDatabase(pdbNewConnect connect, char *name, void* dataRef)
{
	pdatabaseObject dbQuery = npInitDbQuery(connect);
	//pdatabaseObject dbDatabase = npInitDbTable(connect); // Change Now, debug db
	pdatabaseObject dbDatabase = npInitDbDatabase(connect, name, dataRef);

	npSetDbQueryObject(connect, dbQuery, dbDatabase, dataRef);
	npSetDbQueryCommand(connect, dbQuery, Create, dataRef);
	npdbGenQuery(connect, dbQuery, dataRef);
	npdbExecuteQuery(connect, dbQuery, dataRef);

	return 0;
}

int npCreateDatabaseMysql(pdbConnectMysql connect, char *name, void* dataRef)
{
	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject database = npInitMySQLDatabase(name);

	npSetMysqlQueryObject(connect, query, database, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlCREATE, dataRef) == 0)
	{
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
	}

	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, database, dataRef);

	return 0;
}

int npFreeDatabaseObject(pdbNewConnect connect, pdatabaseObject obj, void* dataRef)
{
	switch(obj->type)
	{
	case MySQL:
		npFreeMysqlObject(connect->dbConnection, obj->dbObject, dataRef);
		break;

	case Oracle:
		break;
	}

	//now free the databaseObject
	return 0;
}

int npdbCreateTable(pdbNewConnect connect, char *name, char *fields, void* dataRef)
{
	pdatabaseObject dbQuery = npInitDbQuery(connect);
	pdatabaseObject dbTable = npInitDbTable(connect, name);

	printf("\nnpSetDbQueryObject");
	npSetDbQueryObject(connect, dbQuery, dbTable, dataRef);
	printf("\nnpSetDbQueryCommand");
	npSetDbQueryCommand(connect, dbQuery, Create, dataRef); 
	//npSetDbQueryValues(connect, dbQuery, values, numChunks);
	printf("\nnpSetDbQueryFields");
	npSetDbQueryFields(connect, dbQuery, fields);
	printf("\nnpdbGenQuery");
	npdbGenQuery(connect, dbQuery, dataRef);
	printf("\nnpDbExecuteQuery");
	npdbExecuteQuery(connect, dbQuery, dataRef);

	printf("\nnpFreeDatabaseObject");
	npFreeDatabaseObject(connect, dbQuery, dataRef);
	printf("\nnpFreeDatabaseObject");
	npFreeDatabaseObject(connect, dbTable, dataRef);

	printf("\nReturn");
	return 0;
}

int npCreateTableMysql(pdbConnectMysql connect, char *name, char *fields, void* dataRef)
{
	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject table = npNewInitMySQLTable(name);

	npSetMysqlQueryObject(connect, query, table, dataRef);
	npSetMysqlQueryCommand(connect, query, mysqlCREATE, dataRef);
	printf("\nnodeFields : %s\n", fields);
	npSetMysqlQueryFields(connect, query, fields);
	npGenMysqlQuery(query, dataRef);
	npNewExecuteMysqlQuery(connect, query, dataRef);
	
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	
	return 0;
}

int	npCreateAntzNodeStateTable(pdbNewConnect connect, void* dataRef)
{
	char* nodeFields = malloc(2000);
	printf("\nnpCreateAntzNodeStateTable : %d", connect->dbType);
//	npGenMysqlFields(nodeFields, kNPmapFileBufferMax, kNPnode, dataRef);
	npdbGenFields(connect, nodeFields, kNPnode, dataRef);
	npCreateTable(connect, "nodeStateTable", nodeFields, dataRef);
//	npCreateTableMysql(connect, "nodeStateTable", nodeFields, dataRef); // Get return value on success or failure

	free(nodeFields);

	return 0;
}

int npCreateAntzTagsTable(pdbNewConnect connect, void* dataRef)
{
	pData data = (pData) npGetDataRef();

	char* tagFields = malloc(2000);

	//npGenMysqlFields(tagFields, kNPmapFileBufferMax, kNPtag, data);
	npdbGenFields(connect, tagFields, kNPtag, data);
	npCreateTable(connect, "tagsTable", tagFields, dataRef);

//	npCreateTableMysql(connect, "tagsTable", tagFields, dataRef);

	free(tagFields);
	return 0;
}

int npInsertIntoDbTable(pdbNewConnect connect, char *name, char ***values, int numChunks, void* dataRef)
{
	switch(connect->dbType)
	{
		case MySQL:
			npInsertIntoTableMysql(((struct dbConnectMysql*)connect->dbConnection), name, values, numChunks, dataRef);
			break;
		case Oracle:
			break;
	}


	return 0;
}

pdatabaseObject npInitDbQuery(pdbNewConnect connect)
{
//	pdatabaseObject objectInstance = malloc(sizeof(struct dbObject*));
	pdatabaseObject objectInstance = malloc(sizeof(pdatabaseObject)); // changed this from malloc(sizeof(struct databaseObject)) to sizeof(pdatabaseObject), debug db
	void* object = NULL;
	printf("\nnpInitDbQuery");
	switch(connect->dbType)
	{
		case MySQL:
			printf("\nMySQL");
			objectInstance->type = MySQL;
			printf("\nnpInitMysqlQuery()");
			object = npInitMysqlQuery();
			break;
		case Oracle:
			break;
		default:
			return NULL;
			break;
	}
	
	objectInstance->dbObject = object;
	return objectInstance;
}

pdatabaseObject npInitDbTable(pdbNewConnect connect, char *name)
{
	pdatabaseObject objectInstance = NULL;
	void* object = NULL;
	printf("\nAllocating Object Instance");
//	objectInstance = malloc(sizeof(struct dbObject*));
	objectInstance = malloc(sizeof(struct databaseObject));

	
	printf("\nnpInitDbTable");
	switch(connect->dbType)
	{
		case MySQL:
			objectInstance->type = MySQL;
			object = npNewInitMySQLTable(name);
			break;
		case Oracle:
			break;
		default:
			return NULL;
			break;
	}
	
	objectInstance->dbObject = object;
	return objectInstance;
}

int npSetDbQueryObject(pdbNewConnect connect, pdatabaseObject dbQuery, pdatabaseObject dbTable, void* dataRef)
{
	switch(connect->dbType)
	{
		case MySQL:
			npSetMysqlQueryObject(connect->dbConnection, dbQuery->dbObject, dbTable->dbObject, dataRef);
			break;
		case Oracle:
			break;
	}

	return 0;
}


int npInsertIntoTable(pdbNewConnect connect, char *name, char ***values, int numChunks, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		npInsertIntoTableMysql(connect->dbConnection, name, values, numChunks, dataRef);
		break;
	case Oracle:
		break;
	}

	return 0;
}

int npSetDbQueryCommand(pdbNewConnect connect, pdatabaseObject dbQuery, int command, void* dataRef)
{
	switch(connect->dbType)
	{
		case MySQL:
			npSetMysqlQueryCommand(connect->dbConnection, dbQuery->dbObject, command, dataRef);
			break;
		case Oracle:
			break;
	}

	return 0;
}





int npdbGenQuery(pdbNewConnect connect, pdatabaseObject dbQuery, void* dataRef)
{
	switch(connect->dbType)
	{
		case MySQL:
			npGenMysqlQuery(dbQuery->dbObject, dataRef);
			break;
		case Oracle:
			break;
	}

	return 0;
}

int npdbExecuteQuery(pdbNewConnect connect, pdatabaseObject dbQuery, void* dataRef)
{
	
	switch(connect->dbType)
	{
		case MySQL:
			npNewExecuteMysqlQuery(connect->dbConnection, dbQuery->dbObject, dataRef);
			break;
		case Oracle:
			
			break;
	}

	return 0;
}

// Takes an array of dbQuery structures
struct databaseObject* npInitDbQueries(pdbNewConnect connect, int numOfQueries, pdatabaseObject dbQueryArray)
{
	
	return NULL;
}

// Create, Insert, Select
struct databaseObject* npNewdbTest(pdbNewConnect connect, char *name, char* fields, void* dataRef)
{
	pdatabaseObject dbQueries = NULL;
	pdatabaseObject dbTblCreateQuery = npInitDbQuery(connect);
	pdatabaseObject dbNStable = npInitDbTable(connect, "nodeStateTable");
//	pdatabaseObject dbResults = npInitDbResults(connect);
//	pdatabaseObject dbResult = npInitDbResult(connect, dataRef);
	pdatabaseObject dbResult = NULL;
	
	npSetDbQueryObject(connect, dbTblCreateQuery, dbNStable, dataRef);
	npSetDbQueryCommand(connect, dbTblCreateQuery, Create, dataRef); 
	//npSetDbQueryValues(connect, dbQuery, values, numChunks);
	npSetDbQueryFields(connect, dbTblCreateQuery, fields);
	npdbGenQuery(connect, dbTblCreateQuery, dataRef);
	npdbExecuteQuery(connect, dbTblCreateQuery, dataRef);
	//pause();
	dbResult = npdbStoreResult(connect, dbTblCreateQuery, dataRef);
//	dbResult = npdbStoreResult(
	
	npFreeDatabaseObject(connect, dbTblCreateQuery, dataRef);
	npFreeDatabaseObject(connect, dbNStable, dataRef);
//	dbResults = npdbStoreResults(connect, dbQueries, dbResults, dataRef);
	
	return dbResult;
}

// All mysql functions should only return mysqlObjects containing whatever is necessary, nothing else
// This function might not be necessary, debug db
struct mysqlResult* npInitDbResultMysql(pdbConnectMysql connect, void* dataRef)
{
	pmysqlResult myResult = malloc(sizeof(struct myResult*));
//	MYSQL_RES *result_set;
//	myResult->result = result_set;

	return myResult;
}

struct databaseObject* npInitDbResult(pdbNewConnect connect, void* dataRef)
{
	pdatabaseObject dbResult = malloc(sizeof(struct databaseObject*));
//	dbResult->type = RESULT;
	dbResult->dbObject = malloc(sizeof(struct dbResult*));
	
	switch(connect->dbType)
	{
		case MySQL:
			dbResult->type = MYSQL_RESULT;
		//	((struct dbResult*)dbResult->dbObject)->result = npInitDbResultMysql(connect->dbConnection, dataRef);
			dbResult->dbObject = npInitDbResultMysql(connect->dbConnection, dataRef);
			break;
		case Oracle:
		//	dbResult->TYPE = ORACLE_RESULT;
			break;
	};
	
	return dbResult;
}

struct databaseObject* npdbInsertIntoTable(pdbNewConnect connect, char *name, struct chunksObj *valueChunks, void* dataRef)
{
	pdatabaseObject dbQuery = npInitDbQuery(connect);
	pdatabaseObject dbTable = npInitDbTable(connect, name);
	pdatabaseObject dbResults = npInitDbResults(connect); // This will be passed back up the hierarchy, debug db
//	pdatabaseObject dbResult = npInitDbResult(connect, dataRef); // This should be NULL, until a result exists
	pdatabaseObject dbResult = NULL;

	
	npSetDbQueryObject(connect, dbQuery, dbTable, dataRef); 
	npSetDbQueryCommand(connect, dbQuery, Insert, dataRef); 
	npSetDbQueryValues(connect, dbQuery, valueChunks);
	npdbGenQuery(connect, dbQuery, dataRef);
	npdbExecuteQuery(connect, dbQuery, dataRef);
	
	
	dbResult = npdbStoreResult(connect, dbQuery, dataRef);
	//dbResult = npdbStoreResult(connect, dbResult, dataRef);
//	dbResults = npdbStoreResults(connect, dbResults, dataRef);
	
	//Free, debug db
//	npFreeMysqlObject(connect, dbQuery, dataRef);
//	npFreeMysqlObject(connect, dbTable, dataRef);

	npFreeDbObject(connect->dbConnection, dbQuery, dataRef);
	npFreeDbObject(connect->dbConnection, dbTable, dataRef);

	return dbResult;
}

int npInsertIntoTableMysql(pdbConnectMysql connect, char *name, char ***values, int numChunks, void* dataRef)
{
 	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject table = npNewInitMySQLTable(name);

	npSetMysqlQueryObject(connect, query, table, dataRef);
	npSetMysqlQueryCommand(connect, query, mysqlINSERT, dataRef);
	npSetMysqlQueryValues(connect, query, values);
	printf("\n55 55 55 Generating mysql insert query and executing");
	npGenMysqlQuery(query, dataRef);
	npNewExecuteMysqlQuery(connect, query, dataRef);

	printf("\nFreeing Insert Query");
//	getch();
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	
	return 0;
}

int npdbDropTable(pdbNewConnect connect, char *name, void* dataRef)
{
	pdatabaseObject dbQuery = npInitDbQuery(connect);
	pdatabaseObject dbTable = npInitDbTable(connect, name);

	npSetDbQueryObject(connect, dbQuery, dbTable, dataRef);
	npSetDbQueryCommand(connect, dbQuery, dbInsert, dataRef); 
	npdbGenQuery(connect, dbQuery, dataRef);
	npdbExecuteQuery(connect, dbQuery, dataRef);

	return 0;
}

int npDropTableMysql(pdbConnectMysql connect,char *name, void* dataRef)
{
	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject table = npNewInitMySQLTable(name);

	npSetMysqlQueryObject(connect, query, table, dataRef);
	npSetMysqlQueryCommand(connect, query, mysqlDROP, dataRef);
	npGenMysqlQuery(query, dataRef);
	npNewExecuteMysqlQuery(connect, query, dataRef);

	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);

	return 0;
}

int npDropAntzTagsTable(pdbNewConnect connect, void* dataRef)
{	
	switch(connect->dbType)
	{
		case MySQL:
		npDropTableMysql(connect->dbConnection, "tagsTable", dataRef);
		break;

		case Oracle:
		break;
	}
		
	return 0;
}

int npDropAntzTagsTableMysql(pdbConnectMysql connect, void* dataRef)
{
	npDropTableMysql(connect, "tagsTable", dataRef);
	return 0;
}


int npDropAntzNodeStateTableMysql(pdbConnectMysql connect, void* dataRef)
{
	npDropTableMysql(connect, "nodeStateTable", dataRef);
	return 0;
}

int npSelectTable(pdbNewConnect connect, pdatabaseObject dbQuery, char* name, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		npSelectTableMysql(connect->dbConnection, dbQuery->dbObject, name, dataRef);
		break;
	case Oracle:
		break;
	}

	return 0;
}

int npSelectAntzNodeStateTableMysql(pdbConnectMysql connect, void* dataRef)
{
	pmysqlObject query = npInitMysqlQuery();
	
	npSelectTableMysql(connect, query, "nodeStateTable", dataRef);
//	npSelectTable(connect, 

	((struct mysqlQuery*)query->myobject)->result[0] = npGetResultStructureFromConnectionMysql(connect, dataRef);
	if(((struct mysqlQuery*)query->myobject)->result[0] != NULL)
	{
		((struct mysqlQuery*)query->myobject)->numOfResults++;
	}

	npLoadMysqlResultIntoAntzMysql(connect, query, 1, dataRef);
	
	npFreeMysqlObject(connect, query, dataRef);

	return 0;
}



int npSelectAntzTagsTableMysql(pdbConnectMysql connect, void* dataRef)
{
	pmysqlObject query = npInitMysqlQuery();
	
	npSelectTableMysql(connect, query, "tagsTable", dataRef);
	((struct mysqlQuery*)query->myobject)->result[0] = npGetResultStructureFromConnectionMysql(connect, dataRef);
	if(((struct mysqlQuery*)query->myobject)->result[0] != NULL)
	{
		((struct mysqlQuery*)query->myobject)->numOfResults++;
		npLoadMysqlResultIntoAntzMysql(connect, query, 0, dataRef);
	}
	//npLoadMysqlResultsIntoAntz(connect, query, 0, dataRef);
	
	npFreeMysqlObject(connect, query, dataRef);
	
	return 0;
}


int npSelectTableMysql(pdbConnectMysql connect, pmysqlObject query, char *name, void* dataRef)
{
//	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject table = npNewInitMySQLTable(name);

	printf("\nnpSelectTableMysql\n");
//	getch();
	npSetMysqlQueryObject(connect, query, table, dataRef);
	printf("\nQuery focus : %d", ((struct mysqlQuery*)query->myobject)->focus->type);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		//npSetMysqlQueryConditionals(tagsSelectQuery, WHERE, "id > 10");
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		//npLoadMysqlResultsIntoAntz(query, 1, dataRef);
		//npLoadMysqlResultsIntoAntz(tagsSelectQuery, 0, dataRef); //enum??
	}
	//npFreeMysqlQuery(tagsSelectQuery);
	npFreeMysqlObject(connect, query, dataRef); // Uncomment, debug db
	npFreeMysqlObject(connect, table, dataRef);
	return 0;
}



int npSelectAntzChMapTableMysql(pdbConnectMysql connect, void* dataRef)
{
	pmysqlObject query = npInitMysqlQuery();
	
	npSelectTableMysql(connect, query, "chMapTable", dataRef);
	npLoadMysqlResultIntoAntzMysql(connect, query, 1, dataRef);
	
	npFreeMysqlObject(connect, query, dataRef);
	
	return 0;
}



// get back to
struct chunksObj* npInitChunksObj(struct chunksObj *chunks, int totalObjsSize, int chunkSize, void* dataRef)
{
	int i = 0;
	chunks = NULL;
	//chunks = malloc(sizeof(struct chunksObj*));
	chunks = malloc(sizeof(struct chunksObj));

	printf("\nnpInitChunksObj");
//	getchar();
	if(chunks == NULL)
	{
		printf("\nCannot allocate initial chunk structure");
		//getch();
	}

	chunks->numOfChunks = (totalObjsSize / chunkSize);
	printf("\ntotalObjsSize : %d", totalObjsSize);
	printf("\nchunkSize : %d", chunkSize);
	printf("\nnumOfChunks before : %d", chunks->numOfChunks);
//	getch();
	chunks->numOfChunks++;
	printf("\nnumOfChunks after increment : %d", chunks->numOfChunks);
//	getch();
//	chunks->chunk = malloc(sizeof(struct chunkObj*) * chunks->numOfChunks);	
	chunks->chunk = calloc(chunks->numOfChunks, sizeof(struct chunkObj));
	
	for(i = 0; i < chunks->numOfChunks; i++)
	{		
		printf("%d %p ", i, chunks->chunk[i]);
	}

//	npInitAllChunks(chunks, dataRef);
	//chunks->chunk = malloc(sizeof(struct chunkObj*) * chunks->numOfChunks);

	if(chunks->chunk == NULL)
	{
		printf("\nCannot Allocate chunks->");
		//getch();
	}

	return chunks;
}

void npInitCSVnodes(struct csvNodes *nodes, void *dataRef)
{
	nodes = malloc(sizeof(struct csvNodes*) * 1);

	if(nodes == NULL)
	{
		printf("\nCannot Allocate nodes structure");
		//getch();
	}

	return;
}

void npInitNodesInCSVnodes(struct csvNodes *nodes, int numOfNodes, void* dataRef)
{
	int i = 0;
	nodes->node = malloc(sizeof(struct csvNode) * numOfNodes);
	nodes->numOfNodes = numOfNodes;
	printf("\nnpInitNodesInCSVnodes : numOfNodes : %d\n", nodes->numOfNodes);
	//getch();
//	for(i = 0; i < nodes->numOfNodes; i++)
//		printf("\n%d %p", i, nodes->node[i]);

//	return nodes;
}

struct csvNodes* npAddNodesToCSVnodes(struct csvNodes *nodes, int numOfNodes, void* dataRef)
{
	nodes->node = malloc(sizeof(struct csvNode*) * numOfNodes);

	return nodes;
}

struct chunksObj* npOldAddNodesToChunks(struct chunksObj* chunks, struct csvNodes* nodes, void* dataRef)
{
	int i = 0;
	int v = 0;
	int nodeIndex = 0;
	long size = 0;
	int currentChunkNumber = 0;
	int numOfNodes = 0;
	struct csvNode* tmpNode = NULL;

	for(i = 0; i < nodes->numOfNodes; i++)
	{
		size += nodes->node[i].size;
		//printf("\ntotSize : %d", size);
		if(size >= 65535)
		{
			printf("\nSize : %d", size);
			//getch();
			printf("\nnumOfNodes in chunk %d is %d", currentChunkNumber, numOfNodes);
			printf("\n%d %p",currentChunkNumber, chunks->chunk[currentChunkNumber]);
			printf("\nnpInitNodesInCSVnodes");
	//		chunks->chunk[currentChunkNumber].nodes = npInitNodesInCSVnodes(chunks->chunk[currentChunkNumber].nodes, numOfNodes+1, dataRef);
			npInitNodesInCSVnodes(chunks->chunk[currentChunkNumber].nodes, numOfNodes+1, dataRef);
			printf("\nAfter Malloc");
	//		getch();
			if(chunks->chunk[currentChunkNumber].nodes->node != NULL)
			{
				printf("\ntmpNode is not NULL");
				//getch();
			}
			else
			{
				printf("\ntmpNode is NULL");
				//getch();
			}

	//		chunks->chunk[currentChunkNumber].nodes->node = tmpNode;
			
			chunks->chunk[currentChunkNumber].chunkSize = size;
			printf("\nBefore Loop");
			for(v = numOfNodes; v >= 0; v--)
			{
				
				printf("\nV : %d", v);
				printf("\n(i-v) : %d - %d : %d", i, v, (i-v));
				printf("\nnodeIndex %d", nodeIndex);
				chunks->chunk[currentChunkNumber].nodes->node[nodeIndex] = (nodes->node[i-v]);
				printf("\ncsvStr : %s", chunks->chunk[currentChunkNumber].nodes->node[nodeIndex].csvStr);

				nodeIndex++;
			}
			chunks->chunk[currentChunkNumber].nodes->numOfNodes = nodeIndex;
			
			nodeIndex = 0;
			numOfNodes = 0;
			currentChunkNumber++;
			size = 0;
		}
		else 
		{
			numOfNodes++;
		}

	}

	printf("Does numofNodes == nodes->numOfNodes\n%d == %d", numOfNodes, nodes->numOfNodes);
	//getchar();
	if(numOfNodes == nodes->numOfNodes)
	{
		npInitNodesInCSVnodes(chunks->chunk[currentChunkNumber].nodes, numOfNodes+1, dataRef);
			for(v = numOfNodes; v >= 0; v--)
			{
				printf("\nV : %d", v);
				printf("\n(i-v) : %d - %d : %d", i, v, (i-v));
				printf("\nnodeIndex %d", nodeIndex);
				chunks->chunk[currentChunkNumber].nodes->node[nodeIndex] = (nodes->node[i-v]);
				printf("\ncsvStr : %s", chunks->chunk[currentChunkNumber].nodes->node[nodeIndex].csvStr);
				nodeIndex++;
			}
			chunks->chunk[currentChunkNumber].nodes->numOfNodes = nodeIndex;


		return chunks;
	}
	
	// Now lets take care of the remainder


	printf("\nTake care of the remainder : %d", numOfNodes);
//	getch();
//	getch();
//	x++;
//	pause();
//	 = malloc(sizeof(struct csvNode) * numOfNodes);
/*
	tmpNode = malloc(sizeof(struct csvNode) * numOfNodes);
	chunks->chunk[currentChunkNumber].nodes->node = tmpNode;
	chunks->chunk[currentChunkNumber].chunkSize = size;
*/
//	chunks->chunk[currentChunkNumber].nodes = npInitNodesInCSVnodes(chunks->chunk[currentChunkNumber].nodes, numOfNodes+1, dataRef);
	npInitNodesInCSVnodes(chunks->chunk[currentChunkNumber].nodes, numOfNodes+1, dataRef);

	for(v = numOfNodes; v > 0; v--)
	{
		printf("\nV : %d", v);
		printf("\n(i-v) : %d", (i-v));
		//	printf("\nv-i : %d", v-i);
		printf("\nnumOfNodes : %d", numOfNodes);
		printf("\nnodeIndex : %d", nodeIndex);
		printf("\ncurrentChunkNumber : %d", currentChunkNumber);

		chunks->chunk[currentChunkNumber].nodes->node[nodeIndex] = nodes->node[i-v];
		chunks->chunk[currentChunkNumber].nodes->numOfNodes = nodeIndex;
		printf("\nnodes->node[%d] : %s\n", i-v, nodes->node[i-v].csvStr);
		printf("\nchunks->chunk[%d].nodes->node[%d] : %s\n", currentChunkNumber, nodeIndex, chunks->chunk[currentChunkNumber].nodes->node[nodeIndex].csvStr );
		nodeIndex++;
//		getch();
/*
		if((i-v) == nodes->numOfNodes)
		{
			printf("\nDONE DONE DONE");
			getch();
			break;
		}
*/
/*
		if(nodes->node[i-v].csvStr[0] == '\0')
		{
			numOfNodes = nodeIndex;
			chunks->chunk[currentChunkNumber].nodes->numOfNodes = (numOfNodes-2);
			printf("\nNull Terminated");
			break;
			//pause();
		}
*/
		//	printf("\n%s\n", chunks->chunk[x].nodes->node[v].csvStr);
	}
	

	printf("\nDone Chunking, Last chunk has %d nodes", chunks->chunk[currentChunkNumber].nodes->numOfNodes);
	chunks->chunk[currentChunkNumber].nodes->numOfNodes = nodeIndex;
//	getchar();
	chunks->numOfChunks = currentChunkNumber;
//	getchar();
	return chunks;

}

struct chunksObj* npAddNodesToChunks(struct chunksObj* chunks, struct csvNodes* nodes, void* dataRef)
{
	float numOfChunks = 0;
	int numOfNodes = 0;
	int totalNodeCount = 0;
	int tempNodeCount = 0;
	int i = 0;
	int x = 0;
	int z = 0;
	int tot = 0;
	int size = 0;
//	size += nodes->node[i].size;
	//printf("\nnpAddNodesToChunks");
	//printf("\nNumber of Nodes Total : %d\n", nodes->numOfNodes);
	//getch();
	//printf("\nnodes total size : %d", nodes->totalCsvSize);
	//printf("\npredicted float chunk numbers : %f", (float)(nodes->totalCsvSize)/(float)(65535));
	//printf("\npredicted int chunk numbers : %d", (int)(nodes->totalCsvSize)/(int)(65535));
	numOfChunks = ((float)nodes->totalCsvSize)/((float)65535);
	
	if( ((float)(nodes->totalCsvSize)/(float)(65535)) > ((int)(nodes->totalCsvSize)/(int)(65535)) )
	{
	//	printf("\nfloat value is more than int");
		numOfChunks = ((int)(nodes->totalCsvSize)/(int)(65535));
		numOfChunks++;
	//	getch();
	}

	//printf("\nFinal Chunk Number is %d", (int)numOfChunks);
	//getch();
	numOfNodes = (int)(nodes->numOfNodes)/(int)(numOfChunks);
	if( ((float)(nodes->numOfNodes)/(float)(numOfChunks)) > ((int)(nodes->numOfNodes)/(int)(numOfChunks)) )
	{
	///	numOfNodes++;
		printf("\nNodes in each chunk %d : Total Nodes : %f", numOfNodes, (numOfNodes*numOfChunks));
	//	getch();
	}

	for(i = 0; i < nodes->numOfNodes; i++)
	{
		size += nodes->node[i].size;
		
		if(size >= 65535)
		{
			printf("\nNodes in %d Chunk : %d",z, x);
		//	getchar();
			tot += x;
			npInitNodesInCSVnodes(chunks->chunk[z].nodes, x, dataRef);
			chunks->chunk[z].chunkSize = size;
			x = 0;
			size = 0;
			z++;
		}
		x++;
	}

	//Take care of remainder
	if(size > 0 && size < 65535)
	{
		printf("\n112358 X : %d Left over with size of %d\n", x, size);
		npInitNodesInCSVnodes(chunks->chunk[z].nodes, x, dataRef);
		chunks->chunk[z].chunkSize = size;
	}

	printf("\ntotal nodes : %d %d\n", tot, z);
//	getch();

	for(i = 0; i <= z; i++)
	{
		for(x = 0; x < chunks->chunk[i].nodes->numOfNodes; x++)
		{
			chunks->chunk[i].nodes->node[x] = nodes->node[tempNodeCount];
			tempNodeCount++;
			printf("\ncsvStr : %s\n", chunks->chunk[i].nodes->node[x].csvStr);
		//	getch();
		}
	}

	chunks->numOfChunks = z;
//	getch();
	return chunks;
}

// Make a chunk iterator, debug db
// RIGHT HERE, debug db
struct chunksObj* npABC(struct chunksObj* chunks, struct csvNodes* nodes, void* dataRef)
{
	float numOfChunks = 0;
	int numOfNodes = 0;
	int totalNodeCount = 0;
	int tempNodeCount = 0;
	int i = 0;
	printf("\nnpAddNodesToChunks");
	printf("\nnodes total size : %d", nodes->totalCsvSize);
	printf("\npredicted float chunk numbers : %f", (float)(nodes->totalCsvSize)/(float)(65535));
	printf("\npredicted int chunk numbers : %d", (int)(nodes->totalCsvSize)/(int)(65535));
	numOfChunks = ((float)nodes->totalCsvSize)/((float)65535);
	
	if( ((float)(nodes->totalCsvSize)/(float)(65535)) > ((int)(nodes->totalCsvSize)/(int)(65535)) )
	{
		printf("\nfloat value is more than int");
		numOfChunks = ((int)(nodes->totalCsvSize)/(int)(65535));
		numOfChunks++;
		//getch();
	}
	
	printf("\nFinal Chunk Number is %d", (int)numOfChunks);
	

	numOfNodes = (int)(nodes->numOfNodes)/(int)(numOfChunks);
	if( ((float)(nodes->numOfNodes)/(float)(numOfChunks)) > ((int)(nodes->numOfNodes)/(int)(numOfChunks)) )
	{
	///	numOfNodes++;
		printf("\nNodes in each chunk %d : Total Nodes : %f", numOfNodes, (numOfNodes*numOfChunks));
		//getch();
	}

	for(i = 0; i < (int)numOfChunks; i++)
	{
		totalNodeCount = 0;
		npInitNodesInCSVnodes(chunks->chunk[i].nodes, numOfNodes, dataRef);
		while(totalNodeCount < chunks->chunk[i].nodes->numOfNodes)
		{
			chunks->chunk[i].nodes->node[totalNodeCount] = nodes->node[tempNodeCount]; // tempNodeCount, debug db
			printf("\ncsvStr : %s\n", chunks->chunk[i].nodes->node[totalNodeCount].csvStr);
			totalNodeCount++;
			tempNodeCount++;
			printf("\ntotalNodeCount : %d\n", totalNodeCount);
		}
		printf("\ni : %d\n", i);
	//	getch();
	//	nodes->node[i].size
	//	chunks->chunk[i].nodes->node[nodeIndex]
	}

	//getch();
}

void npInitAllChunks(struct chunksObj *chunks, void* dataRef)
{
	int i = 0;

	printf("\nnumOfChunks : %d\n", chunks->numOfChunks);
	//getch();
//	chunks->chunk = malloc(sizeof(struct chunkObj) * chunks->numOfChunks);
	chunks->chunk = malloc(sizeof(struct chunkObj*) * chunks->numOfChunks);	

	for(i = 0; i < chunks->numOfChunks; i++)
	{		
		printf("%d %p ", i, chunks->chunk[i]);
	}

//	getch();
	return;
}

struct chunksObj* npInitAllChunkNodes(struct chunksObj *chunks, void* dataRef)
{
	int i = 0;

	for(i = 0; i < chunks->numOfChunks; i++)
	{
		chunks->chunk[i].nodes = malloc(sizeof(struct csvNodes*));
		//memset(chunks->chunk[i].nodes->
	}

	return chunks;
}

struct chunksObj* npNewerAllChunk(struct csvNodes *nodes, void* dataRef)
{
	struct chunksObj *chunks = NULL;
	chunks = npInitChunksObj(chunks, nodes->totalCsvSize, 65535, dataRef);
	chunks = npInitAllChunkNodes(chunks, dataRef);
//	npInitNodesInCSVnodes(nodes, nodes->numOfNodes, dataRef);
	chunks = npAddNodesToChunks(chunks, nodes, dataRef);

	return chunks;
}

struct chunksObj* npNewAllChunk(struct csvNodes *nodes, void* dataRef)
{
	struct chunksObj *chunks;
	struct csvNode *tmpNode = NULL;
	int i = 0;
	int currentChunkNumber = 0;
	int numOfNodes = 0;
	int v = 0;
	int nodeIndex = 0;
	unsigned long int size = 0;
	
	chunks = malloc(sizeof(struct chunksObj*));
	chunks->numOfChunks = (nodes->totalCsvSize / 65535);
	chunks->numOfChunks++;
	
	printf("\nTot Chunks : %d", chunks->numOfChunks);
	printf("\nNum Nodes : %d", nodes->numOfNodes);
	printf("\nsizeof(struct csvNode*) == %d", sizeof(struct csvNode*));
//	getch();
	chunks->chunk = malloc(sizeof(struct chunkObj*) * chunks->numOfChunks);
	
	if(chunks->chunk == NULL)
	{
		printf("Cannot Allocate chunks->");
		//pause();
	}
	
	for(i = 0; i < chunks->numOfChunks; i++)
	{
		chunks->chunk[i].nodes = malloc(sizeof(struct csvNodes*));
		if(chunks->chunk[i].nodes == NULL)
		{
			printf("Cannot Allocate nodes");
			//pause();
		}
	}
	
//	pause();
	
	// i - z
	for(i = 0; i < nodes->numOfNodes; i++)
	{
		size += nodes->node[i].size;
		//printf("\ntotSize : %d", size);
		if(size >= 65535)
		{
			printf("\nSize : %d", size);
			printf("\nnumOfNodes in chunk %d is %d", currentChunkNumber, numOfNodes);
			printf("\n%d %p",currentChunkNumber, chunks->chunk[currentChunkNumber]);
	//		tmpNode = malloc(sizeof(struct csvNode*) * numOfNodes);
			chunks->chunk[currentChunkNumber].nodes->node = NULL;
			chunks->chunk[currentChunkNumber].nodes->node = malloc(sizeof(struct csvNode*) * numOfNodes);
			printf("\nAfter Malloc");
		//	getch();
			if(chunks->chunk[currentChunkNumber].nodes->node != NULL)
			{
				printf("\ntmpNode is not NULL");
				//getch();
			}
			else
			{
				printf("\ntmpNode is NULL");
				getch();
			}

	//		chunks->chunk[currentChunkNumber].nodes->node = tmpNode;
			
			chunks->chunk[currentChunkNumber].chunkSize = size;
			printf("\nBefore Loop");
			for(v = numOfNodes; v >= 0; v--)
			{
				
				printf("\nV : %d", v);
				printf("\n(i-v) : %d - %d : %d", i, v, (i-v));

				chunks->chunk[currentChunkNumber].nodes->node[nodeIndex] = (nodes->node[i-v]);
				nodeIndex++;
			}
			chunks->chunk[currentChunkNumber].nodes->numOfNodes = nodeIndex;
			
			nodeIndex = 0;
			numOfNodes = 0;
			currentChunkNumber++;
			size = 0;
		}
		else 
		{
			numOfNodes++;
		}

	}
	
	// Now lets take care of the remainder
	printf("\nTake care of the remainder");
//	x++;
//	pause();
//	 = malloc(sizeof(struct csvNode) * numOfNodes);
	tmpNode = malloc(sizeof(struct csvNode*) * numOfNodes);
	chunks->chunk[currentChunkNumber].nodes->node = tmpNode;
	chunks->chunk[currentChunkNumber].chunkSize = size;
	for(v = numOfNodes; v >= 0; v--)
	{
		printf("\nV : %d", v);
		printf("\n(i-v) : %d", (i-v));
		//	printf("\nv-i : %d", v-i);
		printf("\nnumOfNodes : %d", numOfNodes);
		printf("\nnodeIndex : %d", nodeIndex);
		printf("\ncurrentChunkNumber : %d", currentChunkNumber);
		

			chunks->chunk[currentChunkNumber].nodes->node[nodeIndex] = nodes->node[i-v];
		chunks->chunk[currentChunkNumber].nodes->numOfNodes = nodeIndex;
			printf("\nnodes->node[%d] : %s\n", i-v, nodes->node[i-v].csvStr);	
			printf("\nchunks->chunk[%d].nodes->node[%d] : %s\n", currentChunkNumber, nodeIndex, chunks->chunk[currentChunkNumber].nodes->node[nodeIndex].csvStr );
			nodeIndex++;

		if(nodes->node[i-v].csvStr[0] == '\0')
		{
			numOfNodes = nodeIndex;
			chunks->chunk[currentChunkNumber].nodes->numOfNodes = (numOfNodes-2);
			printf("\nNull Terminated");
			break;
			//pause();
		}
		//	printf("\n%s\n", chunks->chunk[x].nodes->node[v].csvStr);
	}
	
	
	printf("\nDone Chunking");
	return chunks;
}

void npAllChunk(char **buffer, int arbritary)
{
	struct chunksObj *chunks;
	int totalCount = 0;
	int count = 0;
	int totalChunks = 0;
	
	
	
	return;
}

struct csvNodes* npGetTagsValues(void* dataRef)
{
	pData data = (pData) dataRef;
	pNPrecordTag searchTag = NULL;
	struct csvNodes *csvTags; // Rename this type to csvObjs, debug db
	int numOfTags = 0;
	int i = 0;
	int count = 0;
	
	char* title = malloc(100); //?
	char* desc = malloc(100); //?
	
	csvTags = malloc(sizeof(csvTags));
	csvTags->numOfNodes = data->io.gl.hud.tags.recordCount+3; // Rename this to csvTags->numOfObjs, debug db
	csvTags->totalCsvSize = 0; // Rename this to csvTags->totalObjSize, debug db
	csvTags->node = malloc(sizeof(struct csvNode) * csvTags->numOfNodes); // Rename this to csvTags->obj , debug db
	printf("\nnumber of tags : %d", csvTags->numOfNodes);
	
	printf("\nnpNewRetrieveTextTagsValues : recordCount = %d", data->io.gl.hud.tags.recordCount);
	
	printf("\nAllocating space for csvStr");
	for(i = 0; i <= csvTags->numOfNodes; i++)
		csvTags->node[i].csvStr = malloc(sizeof(char) * 1000);
	
	printf("\nTraversing recordTagList");
	for (i=0; i < kNPrecordTagListMax; i++)
	{
		//printf("\ni : %d", i);
		searchTag = data->io.gl.hud.tags.recordList[i];
		
		if(searchTag != NULL)
		{
			printf("\nnumOfTags : %d", numOfTags);
			
			if(searchTag->title != NULL)
			{
				printf("\nEscape Title");
				mysql_escape_string(title, searchTag->title, strlen(searchTag->title));
			}

			if(searchTag->desc != NULL)
			{
				printf("\nEscape Desc");
				mysql_escape_string(desc, searchTag->desc, strlen(searchTag->desc));
			}

			printf("\nSprintf");
			count = sprintf (csvTags->node[numOfTags].csvStr, "%d,%d,%d,\'\"%s\"\',\'\"%s\"\'\n",
						  searchTag->id,					//id is unique to this node
						  searchTag->recordID,		
						  searchTag->tableID,
						  title,
						  desc
						  );
			csvTags->node[numOfTags].size = count;
			csvTags->totalCsvSize += count;
			numOfTags++;
			
		}			
	}
	
	//	mysql_escape_string(escaped, buffer, strlen(buffer));
	
	if(numOfTags == 0)
	{
		printf("\nNo tags exist");
		return NULL;
	}
	
	return csvTags;
}

void npHolder(void* dataRef)
{
	int i = 0;
	//	char* buffer;
	int n = 0;
	pNPrecordTag searchTag = NULL;
	pData data = (pData) dataRef;
	char* valsCopy = malloc(300000);
	int numOfTags = 0;
	struct csvNodes *csvTags;
	//	int numberOfChunks = 0;
	
	char* title = malloc(100); //?
	char* desc = malloc(100); //?
	
	
	csvTags = malloc(sizeof(csvTags));
//	nodes->numOfNodes = data->map.nodeCount;
	csvTags->totalCsvSize = 0;
//	nodes->node = malloc(sizeof(struct csvNode) * nodes->numOfNodes);
	
	//if identical recordID && tableID then replace existing
	printf("\nnpNewRetrieveTextTagsValues : Count = %d", data->io.gl.hud.tags.count);
	//	pause();
	for (i=0; i < kNPrecordTagListMax; i++)
	{
		searchTag = data->io.gl.hud.tags.recordList[i];
		
		if(searchTag != NULL)
		{
			printf("\nnumOfTags : %d", numOfTags);
			numOfTags++;
			
			mysql_escape_string(title, searchTag->title, strlen(searchTag->title));
			mysql_escape_string(desc, searchTag->desc, strlen(searchTag->desc));
			
			n += sprintf ((valsCopy + n), "%d,%d,%d,\'\"%s\"\',\'\"%s\"\'\n",
						  searchTag->id,					//id is unique to this node
						  searchTag->recordID,		
						  searchTag->tableID,
						  title,
						  desc
						  );
			
			
			
		}			
	}
	
	//	mysql_escape_string(escaped, buffer, strlen(buffer));
	
	if(numOfTags == 0)
	{
		printf("\nNo tags exist");
		return; // Was returing null
	}
	
	//increment the buffer
	data->io.write[kNPnode] += n;
	
	return;	 // Was returing csvTags, but only holder function
}

struct chunksObj* npRetrieve(pdbConnectMysql connect, int valueType, void* dataRef)
{
//	pData data = (pData) dataRef;
	char **buffer = NULL;
	char ***chunkedBuffer = NULL;
	struct csvNodes *nodes = NULL;
	struct chunksObj *chunks = NULL;
	
	switch(valueType)
	{
	case CHANNELS:
		buffer = npRetrieveChannelsValues(connect, buffer, dataRef);
		break;
	case CHMAP:
		printf("\nnpNewRetrieveChMapValues(dataRef)");
	//	getch();
		nodes = npNewRetrieveChMapValues(dataRef);
//		chunks = npNewAllChunk(nodes, dataRef);
		chunks = npNewerAllChunk(nodes, dataRef);
		break;
	case NODES:
		//printf("\nnpGetNodesValues2 Before : %i :: %i", data->map.nodeCount, data->map.nodeRootCount);
		//buffer = npGetNodesValues2(buffer, &numberOfChunks, dataRef);
			
		nodes = npFixedNodeValues(dataRef);
//		chunks = npNewAllChunk(nodes, dataRef); // Make this arbritary, debug db
		chunks = npNewerAllChunk(nodes, dataRef);

		printf("\nReturned from chunking function");
	//	getch();
		break;
	case TAGS:
		//	buffer = npGetTagsValues(
		//buffer = npNewGetRetrieveTextTagsValues(dataRef);
		nodes = npGetTagsValues(dataRef);
		if(nodes != NULL)
		{
			chunks = npNewerAllChunk(nodes, dataRef);
		}
		else
		{
			chunks = NULL;
		}
		//chunks = npNewAllChunk(nodes, dataRef);
		
		//chunkedBuffer = npNewChunk(buffer, 64, dataRef);
		break;
	}

	return chunks;
}

// Build out, debug ll
char** npRetrieveChannelsValues(struct dbConnect *connect, char** buffer, void* dataRef)
{
	int i = 0;
	pData data = (pData) dataRef;
	printf("\nnumber of tracks : %d", data->io.ch.numberTracks);
	
	for(i = 0; i < data->io.ch.numberTracks; i++)
	{
		printf("\nMapped track offset %d to attribute %s with channel number %d",
			   data->io.ch.xref[i].trackOffset,
			   data->io.ch.xref[i].attributeName,
			   data->io.ch.xref[i].channelNumber);
	}
	printf("\n\n");
//	readTrackNames(dataRef);
	//pause();
	/*
	data->io.ch.xref[data->io.ch.xrefIndex].attributeName = 
	npNewStrcpy(*(tokens + kNPattributeNameOffset), data);
	data->io.ch.xref[data->io.ch.xrefIndex].trackOffset = 
	atoi(*(tokens + kNPtrackOffset));
	data->io.ch.xref[data->io.ch.xrefIndex].channelNumber = atoi(*(tokens + kNPchannelOffset));
	
	printf("Mapped track offset %d to attribute %s with channel number %d\n",
		   data->io.ch.xref[data->io.ch.xrefIndex].trackOffset,
		   data->io.ch.xref[data->io.ch.xrefIndex].attributeName,
		   data->io.ch.xref[data->io.ch.xrefIndex].channelNumber);
	
	if (data->io.ch.xrefIndex < kNPmaxTrackToAttributeMappings)
		data->io.ch.xrefIndex++;
	 */
	
	return NULL; //for now return null, debug ll
}


int npCreateAntzChMapTable(pdbNewConnect connect, void* dataRef)
{
	char* fields = malloc(2000);
	
	npGenMysqlFields(fields, kNPmapFileBufferMax, kNPchMap, dataRef);
	npCreateTable(connect, "chMapTable", fields, dataRef);
	
	//	npCreateTableMysql(connect, "chMapTable", fields, dataRef);
	
	free(fields);
	
	return 0;
}

int npCreateAntzChannelsTable()
{

	return 0;
}

int npCreateAntzTagsTableMysql(pdbConnectMysql connect, void* dataRef)
{
	char* tagFields = malloc(2000);
	npGenMysqlFields(tagFields, kNPmapFileBufferMax, kNPtag, dataRef);
//	npCreateTable(connect, "nodeStateTable", nodeFields, dataRef);
	npCreateTableMysql(connect, "tagsTable", tagFields, dataRef);
//	npCreateTableMysql(connect, "nodeStateTable", nodeFields, dataRef); // Get return value on success or failure

	free(tagFields);
	return 0;
}

int npCreateAntzChMapTableMysql(pdbConnectMysql connect, void* dataRef)
{
	char* chMapFields = malloc(2000);
	npGenMysqlFields(chMapFields, kNPmapFileBufferMax, kNPchMap, dataRef);
//	npCreateTable(connect, "nodeStateTable", nodeFields, dataRef);
	npCreateTableMysql(connect, "chMapTable", chMapFields, dataRef);
//	npCreateTableMysql(connect, "nodeStateTable", nodeFields, dataRef); // Get return value on success or failure

	free(chMapFields);
	return 0;
}

int npdbGenFields(pdbNewConnect connect, char* fields, int type, void* dataRef)
{
	switch(connect->dbType)
	{
		case MySQL:
			npGenMysqlFields(fields, kNPmapFileBufferMax, type, dataRef);
			break;
		case Oracle:
			break;
	}
	return 0;
}

int npCreateAntzNodeStateTableMysql(pdbConnectMysql connect, void* dataRef)
{
	char* nodeFields = malloc(2000);
	npGenMysqlFields(nodeFields, kNPmapFileBufferMax, kNPnode, dataRef);
//	npCreateTable(connect, "nodeStateTable", nodeFields, dataRef);
	npCreateTableMysql(connect, "nodeStateTable", nodeFields, dataRef);
//	npCreateTableMysql(connect, "nodeStateTable", nodeFields, dataRef); // Get return value on success or failure

	free(nodeFields);

	return 0;
}

int npCreateAntzTableMysql(pdbConnectMysql connect, int tableType, void* dataRef)
{
//	printf("\nnpCreateAntzTableMysql : %d", connect->
	// These will return success or error, error checking, debug ll
	switch(tableType)
	{
	case NODE_STATE_TABLE:
		//npCreateAntzNodeStateTable(connect, dataRef);
		npCreateAntzNodeStateTableMysql(connect, dataRef);
		break;
	case TAGS_TABLE:
		//npCreateAntzTagsTable(connect, dataRef);
		npCreateAntzTagsTableMysql(connect, dataRef);
		break;
	case CHMAP_TABLE:
		//npCreateAntzChMapTable(connect, dataRef);
		npCreateAntzChMapTableMysql(connect, dataRef);
		break;
	case CHANNELS_TABLE:
		npCreateAntzChannelsTable();
		break;	
	}

	return 0;
}

int npAntzTable2(pdbNewConnect connect, int command, int tableType, void* dataRef)
{
	switch(tableType)
	{
	case NODE_STATE_TABLE:
		npCreateAntzNodeStateTable(connect, dataRef);
		break;
	case TAGS_TABLE:
		npCreateAntzTagsTable(connect, dataRef);
		break;
	case CHMAP_TABLE:
		npCreateAntzChMapTable(connect, dataRef);
		break;
	case CHANNELS_TABLE:
		npCreateAntzChannelsTable();
		break;	
	}

	return 0;
}

 
enum {
	ANTZ_DATABASE = 0
};

int npAntzDatabase(pdbNewConnect connect, int command, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		npAntzDatabaseMysql(connect->dbConnection, command, dataRef);
		break;
	case Oracle:
		break;
	}

	return 0;
}

int npAntzDatabaseMysql(pdbConnectMysql connect, int command, void* dataRef)
{
	
	// Error checking, debug ll
	switch(command)
	{
	case Create:
		npCreateAntzDatabaseMysql(connect, dataRef);
		break;
	case Use:
		npUseAntzDatabaseMysql(connect, dataRef);
		break;
	case Drop:
		//npDropAntzDatabaseMysql(connect) // Create this function, debug ll
		break;
	}
	
	return 0;
}

 
//struct databaseObject* npdbCtrl(pdbNewConnect connect, int objectType, int tableType, char *name, char *fields, struct chunksObj *valueChunks, int command, void* dataRef)

int npLoadAntzStateFromDatabase(pdbNewConnect connect, void* dataRef)
{	
	//	npAntzMysql(connect, DATABASE, NULL, Create, dataRef); 
	pdatabaseObject dbResult = npInitDbResult(connect, dataRef);
	pdatabaseObject dbTagsResult = npInitDbResult(connect, dataRef);
	
	dbResult = npdbCtrl(connect, DATABASE, NULL, "Antz", NULL, NULL, NULL, NULL, Use, dataRef);
//	npdbCtrl(connect, DATABASE, NULL, Use, dataRef); // I could eliminate this warning with a little restructureing, debug ll

	
	printf("\nSelect * from nodeStateTable");
	dbResult = npdbCtrl(connect, TABLE, NODE_STATE_TABLE, "nodeStateTable", NULL, NULL, NULL, NULL, Select, dataRef);

//	dbResults = npdbCtrl(connect, TABLE, NODE_STATE_TABLE, "nodeStateTable", NULL, NULL, NULL, NULL, Select, dataRef);
//	npdbCtrl(connect, TABLE, NODE_STATE_TABLE, Select, dataRef);

	dbTagsResult = npdbCtrl(connect, TABLE, TAGS_TABLE, "tagsTable", NULL, NULL, NULL, NULL, Select, dataRef);

//	npdbCtrl(connect, TABLE, TAGS_TABLE, "tagsTable", dataRef);
//	npdbCtrl(connect, TABLE, TAGS_TABLE, Select, dataRef);
	npNewLoadNodeStateResultIntoAntz(dbResult->dbObject, dataRef);

//	npNewLoadTextTags(dbTagsResult->dbObject, dataRef);

	
	
	/*
	switch(connect->dbType)
	{
	case MySQL:
		npLoadAntzStateFromDatabaseMysql(((struct dbConnectMysql*)connect->dbConnection), dataRef);
		break;
	case Oracle:
		break;
	}
	*/

	return 0;	
}

struct databaseObject* npdbTableSelect(pdbNewConnect connect, char *name, char* fields, int conditionalType, char* conditionals, void* dataRef)
{
	pdatabaseObject dbQuery = npInitDbQuery(connect);
	pdatabaseObject dbTable = npInitDbTable(connect, name);
//	pdatabaseObject dbResults = npInitDbResults(connect); // This will be passed back up the hierarchy, debug db
//	pdatabaseObject dbResult = npInitDbResult(connect, dataRef); // This should be NULL, until a result exists
	pdatabaseObject dbResult = NULL;

	
	npSetDbQueryObject(connect, dbQuery, dbTable, dataRef);
	npSetDbQueryCommand(connect, dbQuery, Select, dataRef);
	if(fields != NULL)
	{
		npSetDbQueryFields(connect, dbQuery, fields);
	}
	
	if(conditionals != NULL && conditionalType != NULL)
	{
		npSetDbQueryConditionals(connect, dbQuery, conditionalType, conditionals, dataRef);
	}
	
//	npSetDbQueryValues(connect, dbQuery, valueChunks);
	printf("\nBEFORE\n");
//	getch();
	npdbGenQuery(connect, dbQuery, dataRef);
	printf("\nnpdbExecuteQuery\n");
//	getch();
	npdbExecuteQuery(connect, dbQuery, dataRef);
	
	dbResult = npdbStoreResult(connect, dbQuery, dataRef);
	//dbResult = npdbStoreResult(connect, dbResult, dataRef);
	//	dbResults = npdbStoreResults(connect, dbResults, dataRef);
	
	//Free, debug db
	//These two npFree functions should be npFreeDbObject, debug db THIS IS IT

	
//	npFreeMysqlObject(connect, dbQuery, dataRef);
//	npFreeMysqlObject(connect, dbTable, dataRef);
	
	
	npFreeDbObject(connect->dbConnection , dbQuery, dataRef); // npFreeDbObject currently only takes mysql connections
	npFreeDbObject(connect->dbConnection, dbTable, dataRef);


	return dbResult;
}

struct databaseObject* npdbTableCtrl(pdbNewConnect connect, int objectType, int tableType, char *name, char *fields, struct chunksObj *valueChunks, int conditionalType, char* conditionals, int command, void* dataRef)
{
	pdatabaseObject dbResult = NULL;
//	int numChunks = 0;

	switch(command)
	{
	case Create:
		printf("\nnpdbCreateTable");
		npdbCreateTable(connect, name, fields, dataRef);
		break;

	case Insert:
//		npInsertIntoDbTable(connect, name, values, numChunks, dataRef);
		dbResult = npdbInsertIntoTable(connect, name, valueChunks, dataRef);
		break;

	case Drop:
		break;
			
	case Select:
		//dbResults = npdbSelectTable(connect, name, fields
		printf("\nnpdbTableSelect\n");
		//getch();
		dbResult = npdbTableSelect(connect, name, fields, conditionalType, conditionals, dataRef);
		break;
			
	case Show:
	//  dbResult = npdbTableShow(
		break;
	}

	return dbResult;
}

// Called From 	npdbCtrl(connect, DATABASE, NULL, NULL, NULL, NULL, NULL, NULL, Show, dataRef);
struct databaseObject* npdbShowDatabases(pdbNewConnect connect, void* dataRef)
{
	pdatabaseObject dbQuery = npInitDbQuery(connect);
	pdatabaseObject dbDatabase = npInitDbDatabase(connect, "Antz", dataRef);
	pdatabaseObject dbResult = npInitDbResult(connect, dataRef);
	
	npSetDbQueryObject(connect, dbQuery, dbDatabase, dataRef);
	npSetDbQueryCommand(connect, dbQuery, Show, dataRef); 
	npdbGenQuery(connect, dbQuery, dataRef);
	npdbExecuteQuery(connect, dbQuery, dataRef);
	dbResult = npdbStoreResult(connect, dbQuery, dataRef);
	
	return dbResult;
}

struct databaseObject* npdbDatabaseCtrl(pdbNewConnect connect, int objectType, int tableType, char *name, int command, void* dataRef)
{
	pdatabaseObject result = NULL;
	
	switch(command)
	{
	case Create:
		result = npdbCreateDatabase(connect, name, dataRef);
		break;

	case Use:
		//npUseDatabase(connect, name, dataRef);
		result = npdbUseDatabase(connect, name, dataRef);
		break;

	case Drop:
		
		break;
			
	case Show:
		result = npdbShowDatabases(connect, dataRef);	
		break;
	}


	return result;
}

pdatabaseObject npdbCtrl(pdbNewConnect connect, int objectType, int tableType, char *name, char *fields, struct chunksObj *valueChunks, int conditionalType, char* conditionals, int command, void* dataRef)
{
	pdatabaseObject dbResult = NULL;
	
	switch(objectType)
	{
		case DATABASE:
			dbResult = npdbDatabaseCtrl(connect, objectType, tableType, name, command, dataRef);
			break;
		case TABLE:
			dbResult = npdbTableCtrl(connect, objectType, tableType, name, fields, valueChunks, conditionalType, conditionals, command, dataRef);
			break;
	}

	return dbResult;
}

int npAntzDb(pdbNewConnect connect, int objectType, int tableType, int command, void* dataRef)
{
	printf("\ntype : %d", connect->dbType);
	/*
	switch(objectType)
	{
	case DATABASE:
		npAntzDatabase(connect, command, dataRef);
		//npAntzDatabaseMysql(connect, command, dataRef);
		break;
	case TABLE:
		npAntzTable(connect, command, tableType, dataRef);
		//npAntzTableMysql(connect, command, tableType, dataRef);
		break;
	}
	*/
/*
	switch(connect->dbType)
	{
		case MySQL:
			npAntzMysql(connect->dbConnection, objectType, tableType, command, dataRef);
			break;
		case Oracle:
		  //npAntzOracle
			break;
	}
*/

	return 0;
}


enum {
	ANTZ = 0
}; // Should include DATABASE but redefinition is occuring, debug db

int npRetrieveFrom(int source, int type, void* dataRef)
{
	switch(source)
	{
		case ANTZ:
			break;
		case DATABASE:
		//	npRetrieve(
			break;
	};
	
	return 0;
}
// Implement user permission probing for increased efficency, debug db

int npSaveAntzStateToDatabase(pdbNewConnect connect, void* dataRef)
{
//	int numberOfChunks = 0;
	char* nodeFields = malloc(2000);
	char* tagFields = malloc(2000);
//	char* chMapFields = malloc(2000);
//	char ***chMapValues = NULL;
	struct chunksObj *chMapValues = NULL;
	struct chunksObj *tagChunkedValues = NULL;
	struct chunksObj *chunkedValues = NULL;
	pdatabaseObject dbResult = NULL;
	int i = 0;
	int x = 0;
	//char databaseName[18] = "Antz";
	char *databaseName = malloc(20);

	nposTimeStampCSV(databaseName);
	
	while(databaseName[i] != '.')
	{
		i++;
	}

	printf("\n%s\n", databaseName);
	databaseName[i] = '\0';
	printf("\n%s\n", databaseName);
//	getchar();
	i = 0;

	dbResult = npdbCtrl(connect, DATABASE, NULL, databaseName, NULL, NULL, NULL, NULL, Create, dataRef);
	dbResult = npdbCtrl(connect, DATABASE, NULL, databaseName, NULL, NULL, NULL, NULL, Use, dataRef);

/*
	chMapValues = npRetrieve(connect, CHMAP, dataRef);
	npdbGenFields(connect, chMapFields, kNPchMap, dataRef);
	npdbCtrl(connect, TABLE, CHMAP_TABLE, "chMapTable", chMapFields, NULL, NULL, NULL, Create, dataRef);
	npdbCtrl(connect, TABLE, CHMAP_TABLE, "chMapTable", NULL, chMapValues, NULL, NULL, Insert, dataRef);
*/
	
	/*
	npdbGenFields(connect, tagFields, kNPtag, dataRef);
	tagChunkedValues = npRetrieve(connect, TAGS, dataRef);
	dbResult = npdbCtrl(connect, TABLE, TAGS_TABLE, "tagsTable", tagFields, NULL, NULL, NULL, Create, dataRef);
	if(tagChunkedValues != NULL)
	{
		dbResult = npdbCtrl(connect, TABLE, TAGS_TABLE, "tagsTable", NULL, tagChunkedValues, NULL, NULL, Insert, dataRef);
	}
	*/
	npdbGenFields(connect, nodeFields, kNPnode, dataRef);
	chunkedValues = npRetrieve(connect, NODES, dataRef); // nodeValues no longer required, debug db
	printf("\nAfter npRetrieve");
	//dbResult = npNewdbTest(connect, "nodeStateTable",nodeFields, dataRef);
	//dbResult = npdbCtrl(connect, TABLE, NODE_STATE_TABLE, "nodeStateTable", nodeFields, NULL, Create, dataRef);
	printf("\n0 dbCtrl");
	dbResult = npdbCtrl(connect, TABLE, NODE_STATE_TABLE, "nodeStateTable", nodeFields, NULL, NULL, NULL, Create, dataRef);
	printf("\n1 dbCtrl");
	dbResult = npdbCtrl(connect, TABLE, NODE_STATE_TABLE, "nodeStateTable", NULL, chunkedValues, NULL, NULL, Insert, dataRef);
	printf("\n2 dbCtrl");
//	printf("\nSelect * from nodeStateTable");
//	dbResult = npdbCtrl(connect, TABLE, NODE_STATE_TABLE, "nodeStateTable", NULL, NULL, NULL, NULL, Select, dataRef);

	/*
	npAntzDb(connect, DATABASE, NULL, Create, dataRef); // For creating mysql tables which mirror the csv structure // I could eliminate this warning with a little restructureing, debug ll
	npAntzDb(connect, DATABASE, NULL, Use, dataRef); // I could eliminate this warning with a little restructureing, debug ll

//	npAntzDb(connect, TABLE, TAGS_TABLE, Create, dataRef);
//	npAntzDb(connect, TABLE, TAGS_TABLE, Insert, dataRef);

	npAntzDb(connect, TABLE, NODE_STATE_TABLE, Create, dataRef);
	npAntzDb(connect, TABLE, NODE_STATE_TABLE, Insert, dataRef);

	npAntzDb(connect, TABLE, CHMAP_TABLE, Create, dataRef);
	npAntzDb(connect, TABLE, CHMAP_TABLE, Insert, dataRef); // It seems like I have unnecessary redundancy here, debug ll

	//	npAntzMysql(connect, TABLE, CHANNELS_TABLE, Create, dataRef);
	//	npAntzMysql(connect, TABLE, CHANNELS_TABLE, Insert, dataRef);
*/	


	return 0;
}

int npSetDbQueryConditionals(pdbNewConnect connect, pdatabaseObject dbQuery, int conditionalType, char* conditional, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		//npSetMysqlQueryConditionals(connect->dbConnection, (struct mysqlObject*)(dbQuery->dbObject), conditionalType, conditional, dataRef);
		break;
	case Oracle:
		break;
	}

	return 0;
}

int npSetMysqlQueryConditionals(pdbConnectMysql connect, pmysqlObject query, int conditionalType, char* conditional, void* dataRef)
{
	switch(conditionalType)
	{
		case WHERE:
			strcpy(((struct mysqlQuery*)query->myobject)->conditionals, conditional);
			((struct mysqlQuery*)query->myobject)->conditionalType = conditionalType;
			break;
	}
	
	return 0;
}



char*** npGetRootNodesIds(pdbNewConnect connect, char*** buffer, int interval, void* dataRef)
{

	return NULL;
};

char*** npGetRootNodesIdsFromMysql(pdbConnectMysql connect, char*** buffer, int interval, void* dataRef) // Add buffer
{
	char *conditional = malloc(200);
	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject table = npNewInitMySQLTable("nodeStateTable");

	
	sprintf(conditional, "%s", "parent_id = 0");
	
	npUseAntzDatabaseMysql(connect, dataRef);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		npSetMysqlQueryFields(connect, query, "id");
		npSetMysqlQueryConditionals(connect, query, WHERE, conditional, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		((struct mysqlQuery*)query->myobject)->result[0] = npGetResultStructureFromConnectionMysql(connect, dataRef);
		if(((struct mysqlQuery*)query->myobject)->result[0] != NULL)
		{
			((struct mysqlQuery*)query->myobject)->numOfResults++;
			buffer = npGetResultMysql(connect, ((struct mysqlQuery*)query->myobject)->result[0], buffer, dataRef);
		}
		printf("\nngrnrifm : %s", buffer[0][0]);

		//npLoadMysqlResultsIntoAntz(query, 1, dataRef); //enum??
	}


	//npFreeMysqlQuery(nodeSelectQuery);
	
	free(conditional);
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	return buffer;
}

int npDbGetRootNodes(pdbNewConnect connect, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		break;
	case Oracle:
		break;
	}

	return 0;
}

int npGetRootNodesFromMysql(pdbConnectMysql connect, void* dataRef)
{
	char *conditional = malloc(200);
	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject table = npNewInitMySQLTable("nodeStateTable");
	
		//sprintf(conditional, "%s", "branch_level = 0");
	sprintf(conditional, "%s", "parent_id = 0");
	//	sprintf(conditional, "(record_id >= %d && record_id <= %d) && (branch_level = 1 || branch_level = 0)", from, to);
	
	//	sprintf(conditional, "%s", "record_id % 2 = 0");
	npUseAntzDatabaseMysql(connect, dataRef);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		npSetMysqlQueryConditionals(connect, query, WHERE, conditional, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		npLoadMysqlResultIntoAntzMysql(connect, query, 1, dataRef); //enum??
	}
	//npFreeMysqlQuery(nodeSelectQuery);
	
	free(conditional);
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	return 0;
}

int npDbGetNodeRange(pdbNewConnect connect, char* field, char* from, char* to, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		break;
	case Oracle:
		break;
	}

	return 0;
}

int npGetNodeRangeFromMysql(pdbConnectMysql connect, char* field, char* from, char* to, void* dataRef) 
{
	char *conditional = malloc(200);
	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject table = npNewInitMySQLTable("nodeStateTable");
	
	sprintf(conditional, "%s >= %s && %s <= %s", field, from, field, to);
//	sprintf(conditional, "%s", "record_id = 22564");

//	sprintf(conditional, "(record_id >= %d && record_id <= %d) && (branch_level = 1 || branch_level = 0)", from, to);
	
//	sprintf(conditional, "%s", "record_id % 2 = 0");
	npUseAntzDatabaseMysql(connect, dataRef);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		npSetMysqlQueryConditionals(connect, query, WHERE, conditional, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		npLoadMysqlResultIntoAntzMysql(connect, query, 1, dataRef); //enum??
	}
	//npFreeMysqlQuery(nodeSelectQuery);
	free(conditional);
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	return 0;
}
	

int npGetNodesWithMysql(pdbConnectMysql connect, int fieldType, int fieldValue, void* dataRef) // Might want to make fieldValue a void*
{
	switch(fieldType)
	{
		
	}
	
	return 0;
}

struct csvNodes* npNewRetrieveChMapValues(void* dataRef)
{
	pData data = (pData) dataRef;	
	struct csvNodes *csvChMap;
	int count = 0;
	int i = 0;

	csvChMap = malloc(sizeof(csvChMap));
	csvChMap->numOfNodes = (data->io.ch.xrefIndex + 1); 
	csvChMap->totalCsvSize = 0; 
	csvChMap->node = malloc(sizeof(struct csvNode) * csvChMap->numOfNodes); 
	printf("\nnumber of chMap values : %d", csvChMap->numOfNodes);
	
	for(i = 0; i <= csvChMap->numOfNodes; i++)
		csvChMap->node[i].csvStr = malloc(sizeof(char) * 300);

	printf("\nNumber of channelMapValues :: %d", data->io.ch.xrefIndex);
	
	if(data->io.ch.xrefIndex == 0)
		return NULL;

	printf("\ndata->io.ch.xrefIndex :: %d", data->io.ch.xrefIndex);
	for (i = 0; i < data->io.ch.xrefIndex; i++)
	{
		count = sprintf(csvChMap->node[i].csvStr, "%d,%d,%d,\"%s\",%d,%d,%d\n", i+1, data->io.ch.xref[i].channelNumber, 
						 data->io.ch.xref[i].trackOffset, data->io.ch.xref[i].attributeName, 0, 0, i+1);
		csvChMap->node[i].size = count;
		csvChMap->totalCsvSize += count;
	}
	printf("\ncsvChMap->totalCsvSize : %d", csvChMap->totalCsvSize);// pause();
	
	return csvChMap;
}

char** npRetrieveChMapValues(char** buffer, int *numberOfChunks, void* dataRef)
{
	pData data = (pData) dataRef;		
	int i = 0;
	int count = 0;
	char *valsCopy = malloc(100 * data->io.ch.xrefIndex);
	data->io.write[kNPnode] = valsCopy;

	
	printf("\nNumber of channelMapValues :: %d", data->io.ch.xrefIndex);


	if(data->io.ch.xrefIndex == 0)
		return NULL;
	
	printf("\ndata->io.ch.xrefIndex :: %d", data->io.ch.xrefIndex);
	for (i = 0; i < data->io.ch.xrefIndex; i++)
	{
		count += sprintf((valsCopy + count), "%d,%d,%d,\"%s\",%d,%d,%d\n", i+1, data->io.ch.xref[i].channelNumber, 
						 data->io.ch.xref[i].trackOffset, data->io.ch.xref[i].attributeName, 0, 0, i+1);
	}
	
	printf("\nChannel Map :: -----\n%s\n-----", valsCopy);
	
	buffer = npNewChunkAllValues(valsCopy, numberOfChunks, dataRef);
	
	return buffer;
}

int npSampleNodeStateTableMysql(pdbConnectMysql connect, int mod, void* dataRef)
{
	char*** buffer = NULL;
	
	buffer = npGetRootNodesIdsFromMysql(connect, buffer, 2, dataRef); // npGetRootNodesIDs
	buffer = npNewFilterIDbyInterval(connect, buffer, mod, dataRef);

	npLoadRootNodes(connect, buffer, dataRef);

	npGetChildren(connect, buffer, dataRef);
	//npLoadChildren(buffer, dataRef);
	
	// Free buffer, debug ll
	// npFreeBuffer(void* buffer, int dimensions) ???, debug ll

	return 0;
}

int npSampleNodeStateTable(pdbNewConnect connect, int mod, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		npSampleNodeStateTableMysql(connect->dbConnection, mod, dataRef);
		break;
	case Oracle:
		break;
	}

	return 0;
}

char*** npGetChildNodeFromRecordId(pdbNewConnect connect, char* recordID, void* dataRef)
{
	switch(connect->dbType)
	{
	case MySQL:
		break;
	case Oracle:
		break;
	}

	return NULL;
}

char*** npGetChildNodeFromRecordIdMysql(pdbConnectMysql connect, char* recordID, void* dataRef) // npGetChildNodesFromRecordIdMysql, change name
{
	char*** buffer = NULL;
	char condition[200] = {'\0'};
	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject table = npNewInitMySQLTable("nodeStateTable");
	
	sprintf(condition, "record_id = %s", recordID);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	printf("\nQuery focus : %d", ((struct mysqlQuery*)query->myobject)->focus->type);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		npSetMysqlQueryFields(connect, query, "child_id");
		npSetMysqlQueryConditionals(connect, query, WHERE, condition, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		buffer = npGetResultMysql(connect, ((struct mysqlQuery*)query->myobject)->result[0], buffer, dataRef);
		//npLoadMysqlResultsIntoAntz(query, 1, dataRef);
		//npLoadMysqlResultsIntoAntz(tagsSelectQuery, 0, dataRef); //enum??
	}
	//npFreeMysqlQuery(tagsSelectQuery);
	
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	return buffer;
}

// This could be done more efficiently, select * from nodeStateTable where id IN (x1, x2, x3, x4, x5)
int npLoadRootNodes(pdbConnectMysql connect, char*** buffer, void* dataRef)
{
	char *conditional = malloc(100000); // Make dynamic
	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject table = npNewInitMySQLTable("nodeStateTable");
	int i = 0;
	int count = 0;

	
	//count += sprintf(conditional+count, "%s", "(");
	count += sprintf(conditional+count, "%s", "id IN (");
	while(buffer[i][0][0] != '\0')
	{
		count += sprintf(conditional+count, "%s,", buffer[i][0]);
		i++;
	}

	count--;
	conditional[count] = '\0';
	count += sprintf(conditional+count, "%s", ")");
	
	//printf("\nfound it: %s : %zu\n", conditional, strlen(conditional));
	
	npUseAntzDatabaseMysql(connect, dataRef);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
	//	npSetMysqlQueryFields(query, "id");
		npSetMysqlQueryConditionals(connect, query, WHERE, conditional, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		((struct mysqlQuery*)query->myobject)->result[0] = npGetResultStructureFromConnectionMysql(connect, dataRef);
		((struct mysqlQuery*)query->myobject)->numOfResults = 1;
		npLoadMysqlResultIntoAntzMysql(connect, query, 1, dataRef);
	}
		
	free(conditional);
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	
	return 0;
}

int npGetChildren(pdbConnectMysql connect, char*** buffer, void* dataRef)
{
	char *conditional = malloc(100000);
	pmysqlObject query = npInitMysqlQuery();
	pmysqlObject table = npNewInitMySQLTable("nodeStateTable");
	int i = 0;
	int count = 0;
	
	count += sprintf(conditional+count, "%s", "(");
	while(buffer[i+1][0][0] != '\0')
	{
		count += sprintf(conditional+count, "(%s >= %s && %s <= %s) || ", "id", buffer[i][0], "id", buffer[i+1][0]);
		printf("\nhave not found it : %s", buffer[i][0]);
		//npGetNodeRangeFromMysql(connect, "id", buffer[i][0], buffer[i+1][0], dataRef);
		i++;
	}
	
	count -= 3;
	conditional[count] = '\0';
	count += sprintf(conditional+count, ") %s", " && parent_id != 0");
	
	printf("\nfound it: %s : %zu\n", conditional, strlen(conditional));
	
	npUseAntzDatabaseMysql(connect, dataRef);
	npSetMysqlQueryObject(connect, query, table, dataRef);
	if(npSetMysqlQueryCommand(connect, query, mysqlSELECT, dataRef) == 0)
	{
		//	npSetMysqlQueryFields(query, "id");
		npSetMysqlQueryConditionals(connect, query, WHERE, conditional, dataRef);
		npGenMysqlQuery(query, dataRef);
		npNewExecuteMysqlQuery(connect, query, dataRef);
		((struct mysqlQuery*)query->myobject)->result[0] = npGetResultStructureFromConnectionMysql(connect, dataRef);
		((struct mysqlQuery*)query->myobject)->numOfResults = 1;
		npLoadMysqlResultIntoAntzMysql(connect, query, 1, dataRef);
	}
	
	free(conditional);
	npFreeMysqlObject(connect, query, dataRef);
	npFreeMysqlObject(connect, table, dataRef);
	
	return 0;
}

char*** npNewFilterIDbyInterval(pdbConnectMysql connect, char ***buffer, int interval, void* dataRef)
{
	int i = 0, x = 0, filteredIndex = 0;
	char ***filteredIDs;

	while(buffer[i+1][0][0])
		i++;

	filteredIDs = malloc(sizeof(char*) * i);

	for(x = 0; x < i; x++)
	{
		if(x % interval == 0)
		{
			filteredIDs[filteredIndex] = malloc(sizeof(char*) * 1);
			filteredIDs[filteredIndex][0] = malloc(sizeof(char) * 30);
			strcpy(filteredIDs[filteredIndex][0], buffer[x][0]);
			printf("\nFilteredID %d : %s\n", filteredIndex, filteredIDs[filteredIndex][0]);
			filteredIndex++;
		}
		free(buffer[x][0]);
		free(buffer[x]);
	}

	filteredIDs[filteredIndex] = malloc(sizeof(char*) * 1);
	filteredIDs[filteredIndex][0] = malloc(sizeof(char) * 30);
	filteredIDs[filteredIndex][0][0] = '\0';

//	free(buffer);
	return filteredIDs;
}

char*** npFilterIDbyInterval(pdbConnectMysql connect, char ***buffer, int interval, void* dataRef)
{
	int i = 0;
	int x = 0;
	int rows = 0;
	char ***filteredIDs;
	
	// Null terminate three dimensional array

	
	while ( buffer[i+1][0][0] != '\0')
	{
		i++;
	}
	
	rows = i;
	printf("Rows %d", rows);
	filteredIDs = malloc(sizeof(char*) * (rows));
	
//	for(i = 0; i < rows; i++)
//		filteredIDs[i] = malloc(sizeof(char*) * i); // Might be able to cut this apart by interval

	for(i = 0; i <= rows; i++)
		filteredIDs[i] = malloc(sizeof(char*) * 1); // Might be able to cut this apart by interval

/*
	i++;
	filteredIDs[i] = malloc(sizeof(char*) * 1);
	filteredIDs[i][0] = malloc(sizeof(char) * 2);
	filteredIDs[i][0][0] = '\0';
*/
	filteredIDs[i][0] = malloc(sizeof(char) * 2);
	filteredIDs[i][0][0] = '\0';

	for(i = 0; i < rows; i++)
	{
		if(i % interval == 0)
		{
			filteredIDs[x][0] = malloc(sizeof(char) * 30);
			printf("\nInterval %d : %s", i, buffer[i][0]);
			strcpy(filteredIDs[x][0], buffer[i][0]);
			//strcpy(buffer[i][0], buffer[i][0]);
			x++;
		}
		
	}
	
	printf("\nX is %d", x);
	for(i = 0; i < x; i++)
	{
		printf("\nFiltered ID %d : %s", i, filteredIDs[i][0]);
	}
	
	filteredIDs[x][0] = malloc(sizeof(char) * 30);
	filteredIDs[x][0][0] = '\0';
	
	// Completely free buffer
	for(i = 0; i < rows; i++)
	{
		free(buffer[i][0]);
		free(buffer[i]);
	}
	free(buffer);

	//printf("npFilterIDbyInterval : found null terminator %d", i+1);

	return filteredIDs;
}


 
int npAttachDbToDataRef(pdbNewConnect connect, void* dataRef)
{
	
	return 0;
}

int npAttachMySQLtoDataRef(pdbNewConnect connect, void* dataRef)
{
	pData data = (pData) dataRef;
	data->io.connect = connect;
	return 0;
}

void npNewTest(void *dataRef)
{
	/*
	pData data = (pData) dataRef;
	struct dbConnect *connect = malloc(sizeof(struct dbConnect));

	npInitMySQL(connect, "127.0.0.1", "root", "admin", "test");
	npOpenMySQL(connect, data);
	npAttachMySQLtoDataRef(connect, dataRef);
	*/
	
//	npUseAntzDatabaseMysql(connect, dataRef);
//	npSaveAntzStateToDatabase();
//	npLoadAntzStateFromDatabase(connect, dataRef);
	
//	npSampleNodeStateTable(5);

	// The sample function grabs every nth root node and its children, debug ll
//	npSampleNodeStateTable(connect, 5, dataRef);
//	npLoadAntzTagsTable();

}


struct mysqlObject* npInitMySQLDatabase(char* name)
{
	pmysqlObject object = malloc(sizeof(struct mysqlObject));
	struct mysqlDatabase *tmpDatabase;
	
	object->type = DATABASE;
	printf("\nnpInitMySQLDatabase Type : %d", object->type);
	object->myobject = malloc(sizeof(struct mysqlDatabase));
	tmpDatabase = (struct mysqlDatabase*)object->myobject;
	tmpDatabase->name = malloc(30); //Dynamic
	
	strcpy(tmpDatabase->name, name);
	printf("\nname = %s", ((struct mysqlDatabase*)object->myobject)->name);
	printf("\nname = %s", tmpDatabase->name);
	
	return object;
}

struct mysqlObject* npNewInitMySQLTable(char *name)
{
	pmysqlObject object = malloc(sizeof(struct mysqlObject));
	struct mysqlTable *tmpTable;
	object->type = TABLE;
	object->myobject = malloc(sizeof(struct mysqlTable));
	tmpTable = (struct mysqlTable*)object->myobject;
	
	//-----
	tmpTable->name = malloc(strlen(name)*sizeof(char));
	strcpy(tmpTable->name, name);
	tmpTable->fields = malloc(3000);
	tmpTable->fieldsSize = 3001;
	tmpTable->fields[0] = '\0';
	
	printf("\nfieldSize = %d", ((struct mysqlTable*)object->myobject)->fieldsSize);
	printf("\nfieldSize = %d", tmpTable->fieldsSize);
	//-----

	return object;
}

struct mysqlObject* npInitMysqlQuery()
{
	pmysqlObject queryObject = NULL;
	pmysqlQuery tmpQuery;
//	pmysqlObject object = malloc(sizeof(struct mysqlObject));
//	pmysqlQuery tmpQuery;
	printf("\nAttempting to initialize mysql query : npInitMysqlQuery\n");
	printf("\nAttempting Malloc : %p\n", queryObject);
	queryObject = malloc(sizeof(struct mysqlObject));

	printf("\nAssign Object Type");
	queryObject->type = QUERY;
	printf("\nAssign Object");
	queryObject->myobject = malloc(sizeof(struct mysqlQuery));
	printf("\nSet Temp");
	tmpQuery = (struct mysqlQuery*)queryObject->myobject;
	printf("\nSet Number of Queries");
	((struct mysqlQuery*)queryObject->myobject)->numberOfQueries = 1;
	printf("\nnumberOfQueries : %d", ((struct mysqlQuery*)queryObject->myobject)->numberOfQueries);
	((struct mysqlQuery*)queryObject->myobject)->numOfResults = 0;
	printf("\nnumberOfResults : %d", ((struct mysqlQuery*)queryObject->myobject)->numOfResults);
	((struct mysqlQuery*)queryObject->myobject)->conditionalType = -1; // If set to -1, conditional is not set
	printf("\nconditionalType : %d", ((struct mysqlQuery*)queryObject->myobject)->conditionalType);
	((struct mysqlQuery*)queryObject->myobject)->values = NULL;
	((struct mysqlQuery*)queryObject->myobject)->numOfValues = 0;
	(char*)((struct mysqlQuery*)queryObject->myobject)->fields = NULL;
	//-----
/*
	tmpTable->name = malloc(strlen(name)*sizeof(char));
	strcpy(tmpTable->name, name);
	tmpTable->fields = malloc(3000);
	tmpTable->fieldsSize = 3001;
	tmpTable->fields[0] = '\0';
	
	printf("\nfieldSize = %d", ((struct mysqlTable*)object->myobject)->fieldsSize);
	printf("\nfieldSize = %d", tmpTable->fieldsSize);
*/
	//-----
	
	return queryObject;
}

int npGenMysqlFields(char* fields, int count, int type, void* dataRef)
{
	int i = 0;
	int itemCount = 0;
	
	NPmapTypePtr map = NULL;
	
	pData data = (pData) dataRef;
	
	map = data->map.typeMapNode;		//debug, zz
	

		if (fields == NULL || count < 5)
			return 0;
		
		//set map and item count based on type, could be passed in, debug zz
		switch (type)
		{
			case kNPnode: 
				map = data->map.typeMapNode;
				itemCount = kNPnodeItemCount;
				break;
			case kNPcamera :
				map = data->map.typeMapCamera;
				itemCount = kNPcameraItemCount;
				break;
			case kNPgrid :
				map = data->map.typeMapGrid;
				itemCount = kNPgridItemCount;
				break;
			case kNPpin :
				map = data->map.typeMapPin;
				itemCount = kNPpinItemCount;
				break;
			case kNPtag :
				map = data->map.typeMapTag;
				itemCount = kNPtagItemCount;
				break;
			case kNPchMap :
				map = data->map.typeMapChMap;
				itemCount = kNPChMapItemCount;
				break;
			default : 
				printf ("err 4381 - unknown type, cant write header field names");
				return 0;	
				break;
		}
		
		//	// This seems a little kludgy, debug ll
		//	((struct mysqlTable*)object->myobject)->fields = malloc(itemCount * 70); // sizeof char* or char ???, debug ll
		//	printf("\nbuffer pntr :: %p\n", ((struct mysqlTable*)object->myobject)->fields);
		
		count = 0; 
		for (i=0; i < itemCount; i++)
		{
			// insert a comma for CSV and JSON files
			//first field name to lead without a comma one without comma
			if (i > 0)
				count += sprintf ((fields + count), ",");
			
			//Change INT(10) to INT(11), debug db
			switch (map[i].type)
			{
				case 0 :				// same as default
					count += sprintf ((fields + count), "%s INT(10)", map[i].name );
					break;
				case kNPcharArray :
					count += sprintf ((fields + count), "%s VARCHAR(250)", map[i].name ); // might want to have some sort of setup so that size is variable, debug ll
					break;
				case kNPfloatXYZ : 
					count += sprintf ((fields + count), "%s_x FLOAT(10),%s_y FLOAT(10),%s_z FLOAT(10)",
									  map[i].name, map[i].name, map[i].name );
					break;
				case kNPintXYZ : 
					count += sprintf ((fields + count), "%s_x INT(10),%s_y INT(10),%s_z INT(10)",
									  map[i].name, map[i].name, map[i].name );
					break;
				case kNPboolXYZ : 
					count += sprintf ((fields + count), "%s_x INT(10),%s_y INT(10),%s_z INT(10)",
									  map[i].name, map[i].name, map[i].name );	// Don't know what to do with BOOLS in MySQL, just calling it INT(10) for now, debug ll....should be TINYINT(1)
					break;
				case kNPfloatXYZA : 
					count += sprintf ((fields + count), "%s_x FLOAT(10),%s_y FLOAT(10),%s_z FLOAT(10),%s_a FLOAT(10)",
									  map[i].name, map[i].name, map[i].name, map[i].name );
				case kNPfloatXYZS : 
					count += sprintf ((fields + count), "%s_x FLOAT(10),%s_y FLOAT(10),%s_z FLOAT(10),%s_s FLOAT(10)",
									  map[i].name, map[i].name, map[i].name, map[i].name );
					break;
				case kNPboolXYZS : 
					count += sprintf ((fields + count), "%s_x INT(10),%s_y INT(10),%s_z INT(10),%s_s INT(10)",
									  map[i].name, map[i].name, map[i].name, map[i].name );
					break;
				case kNPubyteRGBA : 
					count += sprintf ((fields + count), "%s_r INT(10),%s_g INT(10),%s_b INT(10),%s_a INT(10)",
									  map[i].name, map[i].name, map[i].name, map[i].name );
					break;
				case kNPubyteRGB : 
					count += sprintf ((fields + count), "%s_r INT(10),%s_g INT(10),%s_b INT(10)",
									  map[i].name, map[i].name, map[i].name );
					break;
				case kNPfloatRGBA : 
					count += sprintf ((fields + count), "%s_r INT(10),%s_g INT(10),%s_b INT(10),%s_a INT(10)",
									  map[i].name, map[i].name, map[i].name, map[i].name );
					break;
				case kNPfloatRGB : 
					count += sprintf ((fields + count), "%s_r INT(10),%s_g INT(10),%s_b INT(10)", 
									  map[i].name, map[i].name, map[i].name );
					break;
					
				default :				// same as case 0
					count += sprintf ((fields + count), "%s INT(10)", map[i].name );
					break;
			}
		}

	
	//	count += sprintf ((((struct mysqlTable*)object->myobject)->fields + count), "\n");
	
	return count;
	
}

void completeConnect()
{
	MYSQL *test;
	test = mysql_init(NULL);

	if(!mysql_real_connect(test, "127.0.0.1", "Lucas", "pass1", "test", 0, NULL, 0))
	{
		printf("\nSomething has gone horribly wrong: Check your login parameters.\n");
	}
	else
	{
		printf("\nConnection Established\n");
	}

	return;
}

void npOpenDbConnectMySQL(pdbNewConnect connection, void* dataRef)
{
	MYSQL *test = NULL;

	printf("\n1");
//	test = mysql_init(NULL);
//	printf("\n2 : %p %p %p", connection, connection->dbConnection, ((struct dbConnectMysql*)connection->dbConnection)->conn  );
	printf("\n2 : %p %p", connection, connection->dbConnection);
	printf("\n%p", ((struct dbConnectMysql*)connection->dbConnection)->conn);
	//((struct dbConnectMysql*)connection->dbConnection)->conn = mysql_init(NULL);
	printf("\nTry to connect : %p", ((struct dbConnectMysql*)connection->dbConnection)->conn );
//	test = mysql_init(NULL);
//	printf("\nReal Try to connect : %p", test );
//	if(!mysql_real_connect(test, "127.0.0.1", "Lucas", "pass1", "test", 0, NULL, 0))
//	{
//		printf("NO NO NO");
//	}
	//((struct dbConnectMysql*)connection->dbConnection)->conn = NULL;
	printf("\nconn: %p", ((struct dbConnectMysql*)connection->dbConnection)->conn);
	printf("\nhost: %s", ((struct dbConnectMysql*)connection->dbConnection)->host);
	printf("\nuser: %s", ((struct dbConnectMysql*)connection->dbConnection)->user);
	printf("\ncpass: %s", ((struct dbConnectMysql*)connection->dbConnection)->pass);

//	 mysql_real_connect(((struct dbConnectMysql*)connection->dbConnection)->conn,
//		((struct dbConnectMysql*)connection->dbConnection)->host, ((struct dbConnectMysql*)connection->dbConnection)->user,
//		((struct dbConnectMysql*)connection->dbConnection)->pass, "test", 0, NULL, 0);
	//connection->conn = mysql_init(NULL);
//	test = mysql_init(NULL);
//	mysql_real_connect(test, "127.0.0.1", "Lucas", "pass1", "test", 0, NULL, 0);
	printf("\nbefore if");
	if(!mysql_real_connect(((struct dbConnectMysql*)connection->dbConnection)->conn,
		((struct dbConnectMysql*)connection->dbConnection)->host, ((struct dbConnectMysql*)connection->dbConnection)->user,
		((struct dbConnectMysql*)connection->dbConnection)->pass, NULL, 0, NULL, 0))
	{
		printf("\nSomething has gone horribly wrong: unable to Establish Connection\n");
	}
	else
	{
		printf("\nConnection Established\n");
	}
/*
	if(((struct dbConnectMysql*)connection->dbConnection)->conn)
	{
		printf("\nConnection Established");
	}
	else
	{
		printf("\nUnable to establish connection");
	}
*/
}

void npOpenDbConnect(pdbNewConnect connection, void* dataRef)
{
	switch(connection->dbType)
	{
	case MySQL:
		printf("\nMySQL");
		npOpenDbConnectMySQL(connection, dataRef); // dbFix, debug ll
		break;
	case Oracle:
		break;
	}
}

// probably don't need char* db
void npInitDbConnection(pdbNewConnect connection, int dbType, char* host, char* user, char* pass, char* db)
{
	connection->dbType = dbType;
	switch (dbType)
	{
	case MySQL:
		npNewInitDbConnectMySQL(connection, host, user, pass, db);
		break;
	case Oracle:
		break;
	}

	return;
}


//------------------------------------------------------------------------------
void npNewInitDbConnectMySQL(pdbNewConnect connection, char* host, char* user, char* pass, char* db)
{
	printf("\nSizeOf* : %d : CHAR* %d", sizeof(struct dbConnectMysql*), sizeof(char*));
	((struct dbConnectMysql*)connection->dbConnection) = (struct dbConnectMysql*)malloc(sizeof(struct dbConnectMysql*));
	printf("\nSizeOf dbConnection : %d", sizeof(struct dbConnectMysql));

	printf("\nHost Length : %d", strlen(host));
	((struct dbConnectMysql*)connection->dbConnection)->host = malloc(strlen(host));
	strcpy(((struct dbConnectMysql*)connection->dbConnection)->host, host);
	printf("\nHost : %s : %d", ((struct dbConnectMysql*)connection->dbConnection)->host, sizeof(((struct dbConnectMysql*)connection->dbConnection)->host));

	printf("\nUser Length : %d", strlen(user));
	((struct dbConnectMysql*)connection->dbConnection)->user = malloc(strlen(user)); //strlen(user)+1
	strcpy(((struct dbConnectMysql*)connection->dbConnection)->user, user);
	printf("\nUser : %s", ((struct dbConnectMysql*)connection->dbConnection)->user);

	printf("\nPre-Malloc");
	printf("\nPass Length : %d", strlen(pass));
	((struct dbConnectMysql*)connection->dbConnection)->pass = malloc(strlen(pass)); //strlen(pass)+1
	printf("\nPre-Pass");
	strcpy(((struct dbConnectMysql*)connection->dbConnection)->pass, pass);
	printf("\nPass : %s", ((struct dbConnectMysql*)connection->dbConnection)->pass);

//	((struct dbConnectMysql*)connection->dbConnection)->db = NULL;
	//((struct dbConnectMysql*)connection->dbConnection)->abc = 5;
//	printf("\nabc : %d", ((struct dbConnectMysql*)connection->dbConnection)->abc);
	((struct dbConnectMysql*)connection->dbConnection)->conn = mysql_init(NULL);
	printf("\nLAST : %p and Size %d", ((struct dbConnectMysql*)connection->dbConnection)->conn, sizeof(((struct dbConnectMysql*)connection->dbConnection)->conn));

	return;
}

void npInitDbConnectMySQL (pdbConnectMysql connection, char* host, char* user, char* pass, char* db)
{	
//	pData data = (pData) dataRef;
	connection->host = malloc(strlen(host)+1);
	strcpy(connection->host, host);

	connection->user = malloc(strlen(user)+1);
	strcpy(connection->user, user);

	printf("\ndatabase username is %s", connection->user);
	printf("\ndatabase password is %s", pass);

	connection->pass = (char*)malloc(strlen(pass)+1);
	printf("\nBefore 2");
	strcpy(connection->pass, pass);

	printf("\nAfter");
//	connection->db = malloc(strlen(db)+1);
//	strcpy(connection->db, db);
	connection->db = NULL;
	
//	connection->q = malloc(sizeof(struct query));
	printf("\nReturning");
	return;
}

void npOpenMySQL (pdbConnectMysql connection, void* dataRef)
{
//	pData data = (pData) dataRef;
//	data->io.npdb.conn = mysql_init(NULL);
	connection->conn = mysql_init(NULL);
	connection->conn = mysql_real_connect(connection->conn, connection->host, "root", connection->pass, NULL, 0, NULL, 0);	
//	data->io.npdb.conn = mysql_real_connect(data->io.npdb.conn, data->io.npdb.host, data->io.npdb.user, data->io.npdb.pass, NULL, 0, NULL, 0);

/*
	if (data->io.npdb.conn == NULL)
	{
		printf("\nUnable to establish connection\n");
	}
	else {
		printf("\nConnection Established\n");
	}
*/
	if (connection->conn == NULL)
		printf("\nUnable to establish connection\n");
	else
		printf("\nConnection Established\n");

	return;
}

//------------------------------------------------------------------------------
void npCloseMySQL (pdbConnectMysql connect, void* dataRef)		//clean-up
{
	return;
}

//------------------------------------------------------------------------------
void npUpdateMySQL (pdbConnectMysql connect, void* dataRef)							//add to ctrl loop, debug zz
{
	return;
}

void npSetMysqlQueryObject(pdbConnectMysql connect, struct mysqlObject* query, pmysqlObject dbOrTable, void* dataRef)
{
//	printf("\n------Begin npSetMysqlQueryObject------ : %d\n", ((struct mysqlQuery*)query->focus));
	((struct mysqlQuery*)query->myobject)->focus = dbOrTable;
	printf("\nnpSetMysqlQueryObject type : %d", ((struct mysqlQuery*)query->myobject)->focus->type);
}

int npSetMysqlQueryCommand(pdbConnectMysql connect, pmysqlObject query, int command, void* dataRef)
{

	// Return error if command and object type doesn't work out, Error Checking debug ll
/*
	switch (((struct mysqlQuery*)query->myobject)->focus->type)
	{
		case DATABASE:
			switch(command)
			{
				case mysqlSELECT:
					printf("\nCannot perform SELECT query on a database");
					return -1;
					break;
				case mysqlINSERT:
					printf("\nCannot perform INSERT query on a database");
					return -1;
					break;
				case mysqlDELETE:
					printf("\nCannot perform DELETE query on a database");
					return -1;
					break;
				case mysqlUPDATE:
					printf("\nCannot perform UPDATE query on a database");
					return -1;
					break;
				case mysqlALTER:
					printf("\nCannot perform ALTER query on a database");
					return -1;
					break;
			}
			break;
		case TABLE:
			switch(command)
			{
			case mysqlUSE:
				printf("\nCannot perform USE query on a table");
				return -1;
				break;
			}
			break;
		case QUERY:
			// Add support for nested queries, debug ll
			break;
	}
	*/
	((struct mysqlQuery*)query->myobject)->command = command;
	return 0;
}

//Check all return values for error checking
void npNewExecuteMysqlQuery(pdbConnectMysql connect, pmysqlObject object, void* dataRef)
{
//	MYSQL_RES *result;
	void *result;
	int i = 0;
	int *ErrResult = 0;
	//int ErrResult = 0;

	printf("npNewExecuteMysqlQuery :: %d", ((struct mysqlQuery*)object->myobject)->numberOfQueries);
	//pause();
	printf("\nQuery : %s", ((struct mysqlQuery*)object->myobject)->query[0]);
	//getch();
	while(((struct mysqlQuery*)object->myobject)->query[i][0] != '\0')
	{
		printf("\nQuery Number : %d", i);
		npExecuteQueryBufferMySQL(connect, ((struct mysqlQuery*)object->myobject)->query[i], ErrResult, dataRef);
		i++;
	}
	printf("\nTotal Queries Executed : %d\n", i);
	//getch();
	/*
	for(i = 0; i < ((struct mysqlQuery*)object->myobject)->numberOfQueries; i++) // I don't need this for loop here, debug db
	{
		printf("\nQuery Number : %d", i);
		npExecuteQueryBufferMySQL(connect, ((struct mysqlQuery*)object->myobject)->query[i], connect, ErrResult, dataRef);
	}
	*/
}

void* npGetResultStructureFromConnectionMysql(pdbConnectMysql connect, void* dataRef)
{
	MYSQL_RES *result;
	int i = 0;
	int ErrResult = 0;

	ErrResult = mysql_errno(connect->conn);
	printf("\nError Number : %d\n", ErrResult);
	result = mysql_store_result(connect->conn);
	if(result == NULL)
	{
		printf("\n NULL");
		return NULL;
	}

	return result;
}

// Seperate out the execution of the query with the result returned
//void npExecuteQueryBufferMySQL(pdbConnectMysql connect, char* buffer, struct dbConnect *connection, int *ErrResult, void* dataRef)
void npExecuteQueryBufferMySQL(pdbConnectMysql connect, char* buffer, int *ErrResult, void* dataRef)

{
	pData data = (pData) dataRef;
	MYSQL_RES *result;
	int isConnected = 0;
	int i = 0;
//	MYSQL *conn = connect->conn;
	int test = 0;
//	int ErrResult = 0;
	int difErrResult = -1;
	printf("\nExec : %s\n", buffer);

//	printf("\nErr Result of Query : %d\n", mysql_query(conn, buffer));
//	printf("\nConnection Pointer : %p", ((struct dbConnectMysql*)data->io.connect->dbConnection)->conn);
	//printf("\nConnection Pointer : %p", ((pdbConnectMysql)data->io.connect->dbConnection)->conn);
	
	//	getch();
	isConnected = mysql_ping(connect->conn);
	printf("\n %d Buffer : %s\n", isConnected, buffer);
//	getch();
	
//	difErrResult = mysql_real_query(((struct dbConnectMysql*)data->io.connect->dbConnection)->conn, buffer, strlen(buffer));
//	difErrResult = mysql_query(((struct dbConnectMysql*)data->io.connect->dbConnection)->conn, "SHOW DATABASES");
//	difErrResult = mysql_query(((struct dbConnectMysql*)data->io.connect->dbConnection)->conn, buffer);
//	difErrResult = mysql_query(connect->conn, buffer);
	difErrResult = mysql_real_query(connect->conn, buffer, strlen(buffer));

	printf("\nReset buffer");
	buffer[strlen(buffer)+1] = '\0';
//	difErrResult = mysql_query(connect->conn, buffer);

	//difErrResult = mysql_query(connect->conn, buffer);
	printf("\ndifErrResult : %d\n", difErrResult);

/*
	result = mysql_store_result(connect->conn);
	if(result == NULL)
	{
		printf("\nnpExecuteQueryBufferMysql NULL");
		return NULL;
	}
	
	return result;
*/
	/*
	result = npGetResultStructureFromConnectionMysql(connect, dataRef);
	return result;
	*/
}

void npLoadMysqlResultIntoAntzMysql(pdbConnectMysql connect, pmysqlObject object, int tableType, void* dataRef)
{
	
	enum{
		TAGS = 0,
		NODES = 1,
		CHMAP = 2,
		CHANNELS = 3
	};
	
//	char *tagsValues;
//	int scanNumRet;
	printf("tableType : %d", tableType);
	switch (tableType)
	{
		case TAGS:
			//tagsValues = parseResult(((struct mysqlQuery*)object->myobject)->result[0], dataRef);
			//printf("\ntagsValues : %s\n", tagsValues);
				//   pause();
			//npLoadTagBuffer(tagsValues, 0, &scanNumRet, dataRef);
			npGetTextTagsFromMySQL(((struct mysqlQuery*)object->myobject)->result[0], dataRef);
			break;
		case NODES:
			npLoadMysqlNodeStateResultsIntoAntz(object, dataRef);
			break;
		case CHMAP:
			break;
		case CHANNELS:
			break;
	}
		
}

int npNewLoadTextTags(struct dbResult *result, void* dataRef)
{
	npGetTextTagsFromMySQL(((struct mysqlResult*)result->resultObj)->result, dataRef);
	return 0;
}

void npGetTextTagsFromMySQL(MYSQL_RES *result, void* dataRef)
{
	pData data = (pData) dataRef;
	pNPrecordTag recordTag = NULL;
//	int id = 0, record_id = 0, table_id = 0;
//	int count = 0;
//	char *buffer = malloc(50);
	MYSQL_ROW row;
	printf("\nnpGetTextTagsFromMySQL LL");
	
	while( row = mysql_fetch_row(result) )
	{
		recordTag = (pNPrecordTag) malloc(sizeof(NPrecordTag)); 
		if (recordTag == NULL)
		{
			printf ("err 4279 - malloc failed making an NPrecordTag\n");
			return;
		}
		npInitDataRecordTag(recordTag, data);
		
		//copy the id fields
		recordTag->id = atoi(row[0]);
		recordTag->recordID = atoi(row[1]);
		recordTag->tableID = atoi(row[2]);
		strcpy(recordTag->title, row[3]);
		strcpy(recordTag->desc, row[4]);		
		printf("\nTitle is %s", recordTag->title);
		printf("\nDescription is %s", recordTag->desc);
	//	pause();
		npAddRecordTag (recordTag, dataRef);
		
	}
	npSyncTags(dataRef);
	
	free(recordTag);

	/*
	if (count > kNPtitleSizeMax)
	{
		printf("err 4959 - record title exceeds kNPtitleSizeMax\n");
		//need to copy then advance buffer
		strncpy(recordTag->title, buffer, kNPtitleSizeMax);
		recordTag->title[kNPtitleSizeMax] = '\0';
		buffer = &buffer[count];
		count = kNPtitleSizeMax;
	}
	else
	{//copy the string and add a null character
		strncpy(recordTag->title, buffer, count);
		recordTag->title[count] = '\0';
		buffer = &buffer[count];
	}
	
	recordTag->titleSize = count;
	printf ("titleSize: %d  ", count);
	printf ("\nrecordTag->title = %s\n", recordTag->title);
	
	buffer++;
	//repeat process for desc
	//next character should be the beginning double quote of the title field
	if (buffer[0] != ',' && buffer[1] != '\"')		//upgrade to allow for a number or no title, debug zz
	{
		printf ("\n");	
		printf ("err 4912 - desc string missing double quote\n");
		
		//permits recordTag with title but no desc
		npAddRecordTag (recordTag, dataRef);
		return recordTag;
	}
	
	buffer++;
	buffer++;
	
	count = 0;
	//will need to convert to "" to '\"'								//debug zz
	while (buffer[count] != '\"' || buffer[count+1] == '\"' )
	{
		if (buffer[count] == '\"' && buffer[count+1] == '\"')
			count++;
		
		count++;
	}
	
	if (count > kNPdescSizeMax)
	{
		printf("err 4960 - record description exceeds kNPdescSizeMax\n");
		//need to copy then advance buffer
		strncpy(recordTag->desc, buffer, kNPdescSizeMax);
		recordTag->desc[kNPdescSizeMax] = '\0';
		buffer = &buffer[count];
		count = kNPdescSizeMax;
	}
	else
	{//copy the string and add a null character
		strncpy(recordTag->desc, buffer, count);
		recordTag->desc[count] = '\0';
		buffer = &buffer[count];
	}
	recordTag->descSize = count;
	printf ("descSize: %d\n", count);
	printf ("\nrecordTag->desc = %s\n", recordTag->desc);
	
	//add the recordTag to the recordTag list
	npAddRecordTag (recordTag, dataRef);
	
	return recordTag;	
	 */
}

void npLoadMysqlNodeStateResultsIntoAntz(pmysqlObject object, void* dataRef)
{	
	int i = 0;
	printf("\nnumOfResults : %d", ((struct mysqlQuery*)object->myobject)->numOfResults);
	for(i = 0; i < ((struct mysqlQuery*)object->myobject)->numOfResults; i++)
	{
		npLoadNodeStateResultIntoAntz(((struct mysqlQuery*)object->myobject)->result[i], dataRef);
	}
	
}

int npNewLoadNodeStateResultIntoAntz(struct dbResult *result, void* dataRef)
{
//	MYSQL_RES* result = result->
//	printf("\n((struct mysqlResult*)result->resultObj)->test = %d", ((struct mysqlResult*)result->resultObj)->test);
	npLoadNodeStateResultIntoAntz(((struct mysqlResult*)result->resultObj)->result, dataRef);
	
	return 0;
}



char*** npGetResultMysql(pdbConnectMysql connect, MYSQL_RES *result, char*** buffer, void* dataRef)
{
	int numFields = 0;
	int numRows = 0;
	unsigned long *lengths;
	int i = 0;
	int x = 0;
	MYSQL_ROW row;

	numFields = mysql_num_fields(result);
	numRows = mysql_num_rows(result);
	buffer = malloc(sizeof(char*) * numRows+1);
	
	printf("\nNumber of rows : %d", numRows);
	for(i = 0; i < numRows; i++)
		buffer[i] = malloc(sizeof(char*) * numFields);
		
	// Null terminate three dimensional array
	buffer[i] = malloc(sizeof(char*) * 1);
	buffer[i][0] = malloc(sizeof(char) * 1);
	buffer[i][0][0] = '\0';
	
	for(i = 0; i < numRows; i++)
	{
		printf("\n");
		row = mysql_fetch_row(result);
		lengths = mysql_fetch_lengths(result);
		for(x = 0; x < numFields; x++)
		{
			buffer[i][x] = malloc((sizeof(char) * lengths[x]) + 150);
			strcpy(buffer[i][x], row[x]); // I might be able to use a memcpy or memmove here to get a speed increase, debug ll
			printf("%s", buffer[i][x]);
			if( x != (numFields - 1) )
				printf(",");
				
		}
	}
	
	if( (buffer[i][0][0]) == '\0' )
		printf("\nfound null terminator");
	
	// Remember to free the result
	mysql_free_result(result);
	
	return buffer;
}

void npLoadNodeStateResultIntoAntz(MYSQL_RES *result, void* dataRef)
{
	MYSQL_ROW row;
	//	int numFields, i = 0;
	int x = 0;
	int count = 0;
	//	buffer = malloc(100000);
	time_t start;
	time_t end;
	pNPnode node = NULL;
	pNPnode nodeParent = NULL;
	int id = 0, type = 0, branchLevel = 0, parentID = 0;
	// Node Data Start
	pData data = (pData) dataRef;
	
	
	// Node Data End
	
	//	memset(buffer, '\0', 1500);
	//numFields = mysql_num_fields(result);
	
	// This could be optimized, strcat needs to traverse to the end of buffer everytime it runs,
	// With each successive loop it takes longer. A better solution would be to simply insert row[i] and ',' from the end each time,
	// thus incurring no performance penalty.
	//	printf("\nFIELDS: %s\n", result->fields); pause();
	start = time(NULL);
	while( row = mysql_fetch_row(result) )
	{
		//printf("\nNew Node", x);
		x++;
		id = atoi(row[0]);
		type = atoi(row[1]);
		branchLevel = atoi(row[5]);
		parentID = atoi(row[4]);
		node = npMapNodeAdd (id, type, branchLevel, parentID, kNPmapNodeCSV, dataRef); // kNPmapNodeCSVvTwo is format		
		
		
		node->selected = atoi(row[3]);
		node->childIndex = atoi(row[7]);
		
		node->chInputID		= atoi(row[9]);
		node->chOutputID	= atoi(row[10]);
		node->chLastUpdated	= atoi(row[11]);
		
		node->average		= atoi(row[12]);
		node->interval		= atoi(row[13]); // Samples???
		
		node->auxA.x		= atoi(row[14]);
		node->auxA.y		= atoi(row[15]);
		node->auxA.z		= atoi(row[16]);
		node->auxB.x		= atoi(row[17]);
		node->auxB.y		= atoi(row[18]);
		node->auxB.z		= atoi(row[19]);
		
		node->colorShift	= (float)atoi(row[20]);
		
		node->rotateVec.x		= (float)atof(row[21]);		//was rotate
		node->rotateVec.y		= (float)atof(row[22]);
		node->rotateVec.z		= (float)atof(row[23]);
		node->rotateVec.angle	= (float)atof(row[24]);	
		
		node->scale.x		= (float)atof(row[25]);
		node->scale.y		= (float)atof(row[26]);
		node->scale.z		= (float)atof(row[27]);
		
		node->translate.x	= atof(row[28]);
		node->translate.y	= atof(row[29]);
		node->translate.z	= atof(row[30]);
		
		node->tagOffset.x	= atof(row[31]);
		node->tagOffset.y	= atof(row[32]);
		node->tagOffset.z	= atof(row[33]);
		
		node->rotateRate.x	= atof(row[34]);
		node->rotateRate.y	= atof(row[35]);
		node->rotateRate.z	= atof(row[36]);
		
		node->rotate.x		= atof(row[37]);					//was rotateRad
		node->rotate.y		= atof(row[38]);
		node->rotate.z		= atof(row[39]);
		
		node->scaleRate.x	= atof(row[40]);
		node->scaleRate.y	= atof(row[41]);
		node->scaleRate.z	= atof(row[42]);
		
		node->translateRate.x = atof(row[43]);
		node->translateRate.y = atof(row[44]);
		node->translateRate.z = atof(row[45]);
		
		node->translateVec.x = atof(row[46]);
		node->translateVec.y = atof(row[47]);
		node->translateVec.z = atof(row[48]);
		
		node->shader		= atoi(row[49]);
		node->geometry		= atoi(row[50]);
		
		node->lineWidth		= atoi(row[51]);
		node->pointSize		= atoi(row[52]);
		node->ratio			= atoi(row[53]);
		
		node->colorIndex	= atoi(row[54]);
		
		node->color.r		= atoi(row[55]);
		node->color.g		= atoi(row[56]);
		node->color.b		= atoi(row[57]);
		node->color.a		= atoi(row[58]);
		
		node->colorFade		= atoi(row[59]);
		node->textureID		= atoi(row[60]);
		
		node->hide			= atoi(row[61]);
		node->freeze		= atoi(row[62]);
		
		//	node->center		= center;		//removed	
		
		node->topo			= atoi(row[63]);			//moved topo
		node->facet			= atoi(row[64]);		//added topo facet number
		
		node->autoZoom.x	= atoi(row[65]);	//moved down a slot
		node->autoZoom.y	= atoi(row[66]);
		node->autoZoom.z	= atoi(row[67]);
		
		//	node->scroll		= scroll;		//removed made space for facet, zz debug
		
		node->triggerHi.x	= atoi(row[68]);
		node->triggerHi.y	= atoi(row[69]);
		node->triggerHi.z	= atoi(row[70]);
		
		node->triggerLo.x	= atoi(row[71]);
		node->triggerLo.y	= atoi(row[72]);
		node->triggerLo.z	= atoi(row[73]);
		
		node->setHi.x		= atof(row[74]);
		node->setHi.y		= atof(row[75]);
		node->setHi.z		= atof(row[76]);
		
		node->setLo.x		= atof(row[77]);
		node->setLo.y		= atof(row[78]);
		node->setLo.z		= atof(row[79]);
		
		node->proximity.x	= atof(row[80]);
		node->proximity.y	= atof(row[81]);
		node->proximity.z	= atof(row[82]);
		
		node->proximityMode.x = atoi(row[83]);			//int cast supports 1st ver CSV
		node->proximityMode.y = atoi(row[84]);
		node->proximityMode.z = atoi(row[85]);
		
		node->segments.x	= atoi(row[86]);		//grid segments were stored in node->data,
		node->segments.y	= atoi(row[87]);		//was node->data->segments.x
		node->segments.z	= atoi(row[88]);		//now node->segments.x
		
		node->tagMode		= atoi(row[89]);
		node->formatID		= atoi(row[90]);
		node->tableID		= atoi(row[91]);
		node->recordID		= atoi(row[92]);
		//node->size		= lineSize;				// handled during node creation
		
		//support for first version CSV
		//	if (format == kNPmapNodeCSVvOne)
		//	npMapCSVvOne (node);
		
		//file compatability prior to 2012-04-22
		if (node->topo == 0 && node->type == kNodePin)
		{
			//printf ("topo = 0   id: %d\n", node->id);
			if (node->branchLevel == 0)
				node->topo = kNPtopoPin;	//set root topo to a pin
			else if (node->parent != NULL)  //orhpan child methods in npMapSort
			{
				nodeParent = node->parent;
				if ( nodeParent->topo == kNPtopoPin || nodeParent->topo == 0
					|| nodeParent->topo == kNPtopoTorus )
					node->topo = kNPtopoTorus;
				else
					node->topo = kNPtopoPin;
			}
			
			
			
		}
		
	}
	//buffer[count] = '\0';
	end = time(NULL);
	printf("\nDifference TIME : %f\n", difftime(end, start));
	//	printf("\nNumber of nodes added : %d", x);
	//	printf("\nDONE PARSING : %s\n", buffer);// pause();
	//printf("\nnpParseResultMySQL :: \n%s\n", buffer);
	
	
	if(result != NULL)
	{
		mysql_free_result(result);
	}
	//	return buffer;
}

void npGenDbQuery(pdatabaseObject object, void* dataRef)
{

}

void npGenMysqlQuery(pmysqlObject object, void* dataRef)
{
	pmysqlObject focused = ((struct mysqlQuery*)object->myobject)->focus;
	printf("\nnpGenMysqlQuery type : %d", focused->type);
//	printf("\nnpGenMysqlQuery type 2 : %d", object->type);
	

	switch (focused->type)
	{
		case DATABASE:
		//	sprintf(query, "%s DATABASE %s", );
			npGenMysqlDatabaseQuery(object);
			break;
		case TABLE:
			npGenMysqlTableQuery(object, dataRef);
			break;
		case QUERY:
			
			printf("\nINVALID REQUEST\n");
			break;
	}
}

void npNewGenMysqlTableQueryInsert(pmysqlObject object, void* dataRef)
{
	struct chunksObj *valuesChunks = ((struct mysqlQuery*)object->myobject)->values;
	pmysqlObject focused = ((struct mysqlQuery*)object->myobject)->focus->myobject;
	int i = 0;
	int x = 0;
	int count = 0;
	
	printf("\nThere are %d chunks", valuesChunks->numOfChunks);
	getch();
	for(i = 0; i < valuesChunks->numOfChunks+1; i++) //added -1, debug db
	{
		printf("\nI : %d", i);
		count = sprintf(((struct mysqlQuery*)object->myobject)->query[i], "INSERT INTO %s VALUES ", ((struct mysqlDatabase*)focused)->name); // This really only needs to be done once, debug db
		printf("\nsprintf, numOfNodes : %d", valuesChunks->chunk[i].nodes->numOfNodes);
		if(i == 228)
		{
			//getch();
		}
		
		for(x = 0; x < valuesChunks->chunk[i].nodes->numOfNodes; x++) // removed <=, debug db
		{
			printf("\nX : %d", x);
			count += sprintf(((struct mysqlQuery*)object->myobject)->query[i]+count, "(%s),", valuesChunks->chunk[i].nodes->node[x].csvStr);
		}
		((struct mysqlQuery*)object->myobject)->query[i][count-1] = '\0';
		printf("\nQUERY : %s", ((struct mysqlQuery*)object->myobject)->query[i]);
	//	getchar();
	//	printf("\nSize of chunk %d", valuesChunks->chunk[i].chunkSize);
		count = 0;
	}
	((struct mysqlQuery*)object->myobject)->query[i][0] = '\0';

	printf("\nDONE");
	//pause();

}

// Make a print chunks function, debug db
// RIGHT NOW URGENT, debug db
// need to change setValue function, debug db
void npOldGenMysqlTableQueryInsert(pmysqlObject object, void* dataRef)
{
	int i = 0;
	int x = 0;
	int size = 0;
	int chunkSize = 0;

	struct chunksObj *valuesChunks = ((pmysqlQuery)object->myobject)->values;

	printf("\nThere are %d chunks", valuesChunks->numOfChunks);
	getch();

	for(i = 0; i <= valuesChunks->numOfChunks; i++)
	{
		printf("\nChunk : %d\n",i);
		for(x = 0; x < valuesChunks->chunk[i].nodes->numOfNodes; x++)
		{
			printf("\nNode Number : %s\n", valuesChunks->chunk[i].nodes->node[x].csvStr);
			size += strlen(valuesChunks->chunk[i].nodes->node[x].csvStr);
			chunkSize += strlen(valuesChunks->chunk[i].nodes->node[x].csvStr);
			//getch();
		}
		printf("\nThat was Chunk : %d\n",i);
		printf("\nChunk Size: %d : %d\n", chunkSize, valuesChunks->chunk[i].chunkSize);
		chunkSize = 0;
		getch();
	}
	printf("\nDone Printing Chunks, Total Size is %d\n", size);
	getch();
}

void npGenMysqlTableQuery(pmysqlObject object, void* dataRef)
{
	// Check to make sure mysqlObject is of type query
	// Add options later for selecting database options like engines
	
	// Not sure if USE or SHOW belongs here
//	FILE *f;
//	int numOfQueries = 0;
	int count = 0;
	int i = 0;
	int x = 0;
//	int y = 0;
	int z = 0;
	
//	char buffer[500000] = {'\0'};
	// Check that focused actually points somewhere
	pmysqlObject focused = ((struct mysqlQuery*)object->myobject)->focus->myobject;
	
	
	printf("\nCOMMAND IS %d\n",((struct mysqlQuery*)object->myobject)->command);

	switch (((struct mysqlQuery*)object->myobject)->command)
	{
		case Alter:
			break;
			
		case Create:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlTable*)focused)->name);
			printf("\nTABLE FIELDS ARE %s\n", ((struct mysqlQuery*)object->myobject)->fields);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "CREATE TABLE %s (%s)", ((struct mysqlTable*)focused)->name, ((struct mysqlQuery*)object->myobject)->fields);
			((struct mysqlQuery*)object->myobject)->query[1][0] = '\0';
			printf("\nnpGenMysqlTableQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			//	pause();
			break;
			
		case Drop:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "DROP TABLE %s", ((struct mysqlTable*)focused)->name);
			((struct mysqlQuery*)object->myobject)->query[1][0] = '\0';
			printf("\nnpGenMysqlTableQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			break;
			
		case Insert:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			npNewGenMysqlTableQueryInsert(object, dataRef);
/*
			while(((struct mysqlQuery*)object->myobject)->values[i][x] != '\0')
			{
				printf("\nI GEN : %d", i);
				//getch();
				count += sprintf(((struct mysqlQuery*)object->myobject)->query[z], "INSERT INTO %s VALUES ", ((struct mysqlDatabase*)focused)->name);

				while(((struct mysqlQuery*)object->myobject)->values[i][x] != '\0') // Never equals '\0', debug db
				{
			//		sprintf(((struct mysqlQuery*)object->myobject)->query[z], "INSERT INTO %s VALUES %s", ((struct mysqlDatabase*)focused)->name, ((struct mysqlQuery*)object->myobject)->values[i][x]);
					count += sprintf((((struct mysqlQuery*)object->myobject)->query[z] + count), "(%s),", ((struct mysqlQuery*)object->myobject)->values[i][x]);
					printf("\nVals %d %d : %s",i,x, ((struct mysqlQuery*)object->myobject)->values[i][x]);
					printf("\nCount : %d", count);
					x++;	
					//	getch();

				}
				((struct mysqlQuery*)object->myobject)->query[z][count-1] = '\0';
				printf("\nnpGenMysqlTableQuery %d : %s\n",z, ((struct mysqlQuery*)object->myobject)->query[z]);
			//	getch();
				count = 0; //debug db
				x = 0;
				z++;
				i++;

				
			}
			((struct mysqlQuery*)object->myobject)->query[z][0] = '\0';
			//printf("\n---\n%s\n---\n", ((struct mysqlQuery*)object->myobject)->query[z][0]);
		//	getch();
*/
			break;
			
		case Select:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			
			switch (((struct mysqlQuery*)object->myobject)->conditionalType)
			{
				case WHERE:
					if(((struct mysqlQuery*)object->myobject)->fields == NULL)
					{
					sprintf(((struct mysqlQuery*)object->myobject)->query[0], "SELECT * FROM %s WHERE %s", ((struct mysqlDatabase*)focused)->name, ((struct mysqlQuery*)object->myobject)->conditionals);
					printf("WHERE : %s", ((struct mysqlQuery*)object->myobject)->query[0]);
					}
					else {
						sprintf(((struct mysqlQuery*)object->myobject)->query[0], "SELECT %s FROM %s WHERE %s",((struct mysqlQuery*)object->myobject)->fields ,((struct mysqlDatabase*)focused)->name, ((struct mysqlQuery*)object->myobject)->conditionals);
						printf("WHERE : %s", ((struct mysqlQuery*)object->myobject)->query[0]);
					}

					//pause();
					break;
					
				case ORDERBY:
					break;
					
				case -1:
					sprintf(((struct mysqlQuery*)object->myobject)->query[0], "SELECT * FROM %s", ((struct mysqlDatabase*)focused)->name);
					((struct mysqlQuery*)object->myobject)->query[1][0] = '\0';
					break;
				
			}
			
			break;
			
		case Use:
			break;
			
		case Show:
			break;
			
	}


/*
	switch (((struct mysqlQuery*)object->myobject)->command)
	{
		case mysqlALTER:
			break;
			
		case mysqlCREATE:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlTable*)focused)->name);
			printf("\nTABLE FIELDS ARE %s\n", ((struct mysqlQuery*)object->myobject)->fields);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "CREATE TABLE %s (%s)", ((struct mysqlTable*)focused)->name, ((struct mysqlQuery*)object->myobject)->fields);
			printf("\nnpGenMysqlTableQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			//	pause();
			break;
			
		case mysqlDROP:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "DROP TABLE %s", ((struct mysqlTable*)focused)->name);
			printf("\nnpGenMysqlTableQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			break;
			
		case mysqlINSERT:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			for(i = 0; i < ((struct mysqlQuery*)object->myobject)->numberOfQueries; i++)
			{
				sprintf(((struct mysqlQuery*)object->myobject)->query[i], "INSERT INTO %s VALUES %s", ((struct mysqlDatabase*)focused)->name, ((struct mysqlQuery*)object->myobject)->values[i]);
				//printf("\nnpGenMysqlTableQuery %d : %s\n",i, ((struct mysqlQuery*)object->myobject)->query[i]);
			}
			break;
			
		case mysqlSELECT:
			printf("\nTABLE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			
			switch (((struct mysqlQuery*)object->myobject)->conditionalType)
			{
				case WHERE:
					if(((struct mysqlQuery*)object->myobject)->fields == NULL)
					{
					sprintf(((struct mysqlQuery*)object->myobject)->query[0], "SELECT * FROM %s WHERE %s", ((struct mysqlDatabase*)focused)->name, ((struct mysqlQuery*)object->myobject)->conditionals);
					printf("WHERE : %s", ((struct mysqlQuery*)object->myobject)->query[0]);
					}
					else {
						sprintf(((struct mysqlQuery*)object->myobject)->query[0], "SELECT %s FROM %s WHERE %s",((struct mysqlQuery*)object->myobject)->fields ,((struct mysqlDatabase*)focused)->name, ((struct mysqlQuery*)object->myobject)->conditionals);
						printf("WHERE : %s", ((struct mysqlQuery*)object->myobject)->query[0]);
					}

					//pause();
					break;
					
				case ORDERBY:
					break;
					
				case -1:
					sprintf(((struct mysqlQuery*)object->myobject)->query[0], "SELECT * FROM %s", ((struct mysqlDatabase*)focused)->name);
					break;
				
			}
			
			break;
			
		case mysqlUSE:
			break;
			
		case mysqlSHOW:
			break;
			
	}
*/	
}

void npGenMysqlDatabaseQuery(pmysqlObject object)
{
	// Check to make sure mysqlObject is of type query
	// Add options later for selecting database options like engines
	
	// Not sure if USE or SHOW belongs here

	
	char buffer[50] = {'\0'};
	// Check that focused actually points somewhere
	pmysqlObject focused = ((struct mysqlQuery*)object->myobject)->focus->myobject;


	printf("\nCOMMAND IS %d\n",((struct mysqlQuery*)object->myobject)->command);
/*
	switch (((struct mysqlQuery*)object->myobject)->command)
	{
		case mysqlALTER:
			break;
			
		case mysqlCREATE:
			printf("\nDATABASE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "CREATE DATABASE %s", ((struct mysqlDatabase*)focused)->name);
			printf("\nnpGenMysqlDatabaseQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
		//	pause();
			break;
		
		case mysqlDROP:
			printf("\nDATABASE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "DROP DATABASE %s", ((struct mysqlDatabase*)focused)->name);
			printf("\nnpGenMysqlDatabaseQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			break;
			
		case mysqlUSE:
			printf("\nDATABASE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "USE %s", ((struct mysqlDatabase*)focused)->name);			
			printf("\nnpGenMysqlDatabaseQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			break;
			
		case mysqlSHOW:
			break;
			
	}
	*/

		switch (((struct mysqlQuery*)object->myobject)->command)
	{
		case Alter:
			break;
			
		case Create:
			printf("\nDATABASE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "CREATE DATABASE %s", ((struct mysqlDatabase*)focused)->name);
			((struct mysqlQuery*)object->myobject)->query[1][0] = '\0';
			printf("\nnpGenMysqlDatabaseQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
		//	pause();
			break;
		
		case Drop:
			printf("\nDATABASE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "DROP DATABASE %s", ((struct mysqlDatabase*)focused)->name);
			((struct mysqlQuery*)object->myobject)->query[1][0] = '\0';
			printf("\nnpGenMysqlDatabaseQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			break;
			
		case Use:
			printf("\nDATABASE NAME IS %s\n", ((struct mysqlDatabase*)focused)->name);
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "USE %s", ((struct mysqlDatabase*)focused)->name);			
			((struct mysqlQuery*)object->myobject)->query[1][0] = '\0';
			printf("\nnpGenMysqlDatabaseQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			break;
			
		case Show:
			sprintf(((struct mysqlQuery*)object->myobject)->query[0], "SHOW %s", "DATABASES");
			printf("\nnpGenMysqlDatabaseQuery : %s\n", ((struct mysqlQuery*)object->myobject)->query[0]);
			((struct mysqlQuery*)object->myobject)->query[1][0] = '\0';
			break;
			
	}
}

void npSetDbQueryFields(pdbNewConnect connect, pdatabaseObject object, char *fields)
{
	pmysqlObject obj = object->dbObject; // ???, debug db
	switch(connect->dbType)
	{
		case MySQL:
			//object->dbObject->myobject->fields = fields;
			((struct mysqlQuery*)obj->myobject)->fields = fields;
			break;
		case Oracle:		
			break;
	}

//	return 0;
}

void npSetMysqlQueryFields(pdbConnectMysql connect, pmysqlObject object, char *fields)
{
	//Check to see if object type supports fields
	((struct mysqlQuery*)object->myobject)->fields = fields;
	
}


int npSetDbQueryValues(pdbNewConnect connect, pdatabaseObject object, struct chunksObj *valueChunks)
{
	switch(connect->dbType)
	{
		case MySQL:
			npSetMysqlQueryValues(connect->dbConnection, object->dbObject, valueChunks);
			break;
		case Oracle:		
			break;
	}


	return 0;
}

void npSetMysqlQueryValues(pdbConnectMysql connect, pmysqlObject object, struct chunksObj *valueChunks)
{
	//buffer = npNewChunkAllValues(valsCopy, numberOfChunks, dataRef);

	((struct mysqlQuery*)object->myobject)->values = valueChunks;
	//((struct mysqlQuery*)object->myobject)->numberOfQueries = numberOfChunks; // Choose one, numberOfChunks or numberOfQueries???
	//printf("NUMBER OF QUERIES IS %d", ((struct mysqlQuery*)object->myobject)->numberOfQueries);
	//pause();
}

char** npFinalFormatChunks(char **buffer, int *numberOfChunks)
{
	char *values = malloc(sizeof(char) * 300000);
	char *formattedValues = malloc(sizeof(char) * 300000); 
	int x = 0;

	for(x = 0; x < *numberOfChunks; x++)
	{
		npFormatChunk(buffer[x], formattedValues, values);
	//	printf("\n CHUNKnum : %d \nformattedValues : %s\n ", x, formattedValues);
		printf("Size of formattedValues : %zu", strlen(formattedValues));

		if(strlen(formattedValues) == 0)
		   (*numberOfChunks)--;
		
		strcpy(buffer[x], formattedValues);
	}		
	
	return buffer;
}
char** npNewGetRetrieveTextTagsValues(void* dataRef)
{
	pData data = (pData) dataRef;
	char **myBuffer = NULL;
	pNPrecordTag searchTag = NULL;
	int i = 0;
//	int x = 0;
//	int n = 0;
	int numOfTags = 0;
	char* title = malloc(100); //?
	char* desc = malloc(100); //?

	myBuffer = malloc(sizeof(char*) * 40000); // hack, debug db

	printf("\nnpNewRetrieveTextTagsValues : Count = %d", data->io.gl.hud.tags.count);
	printf("\nnpNewRetrieveTextTagsValues : recordCount = %d", data->io.gl.hud.tags.recordCount);

	//pause();
	
	for (i=0; i < kNPrecordTagListMax; i++)
	{
		searchTag = data->io.gl.hud.tags.recordList[i];
		if(searchTag != NULL)
		{
			myBuffer[numOfTags] = malloc(sizeof(char) * 1000);
			mysql_escape_string(title, searchTag->title, strlen(searchTag->title));
			mysql_escape_string(desc, searchTag->desc, strlen(searchTag->desc));
			
			sprintf ((myBuffer[numOfTags]), "%d,%d,%d,\'\"%s\"\',\'\"%s\"\'\n",
						  searchTag->id,					//id is unique to this node
						  searchTag->recordID,		
						  searchTag->tableID,
						  title,
						  desc
						  );
			printf("\nNew Tag : %s\n", myBuffer[numOfTags]);
			numOfTags++;
			
		}
	}
		myBuffer[numOfTags-1] = malloc(sizeof(char) * 2);
		myBuffer[numOfTags-1][0] = '\0';

	if(numOfTags == 0)
	{
		printf("\nNo tags exist");
		return NULL;
	}
	else {

	}

	return myBuffer;
}

char** npGetRetrieveTextTagsValues(char** buffer, int *numberOfChunks, int debug, void* dataRef)
{
	int i = 0;
	//	char* buffer;
	int n = 0;
	pNPrecordTag searchTag = NULL;
	pData data = (pData) dataRef;
	char* valsCopy = malloc(300000);
	int numOfTags = 0;
//	int numberOfChunks = 0;
	
	char* title = malloc(100); //?
	char* desc = malloc(100); //?
	
	//if identical recordID && tableID then replace existing
	printf("\nnpNewRetrieveTextTagsValues");
	//	pause();
	for (i=0; i < kNPrecordTagListMax; i++)
	{
		searchTag = data->io.gl.hud.tags.recordList[i];
		
		if(searchTag != NULL)
		{
			numOfTags++;
			if(debug == 1)
			{
				printf("\nFound New Tag");
				printf("\nsearchTag->id == %d", searchTag->id);
				printf("\nsearchTag->recordID == %d", searchTag->recordID);
				printf("\nsearchTag->tableID == %d", searchTag->tableID);
				printf("\nsearchTag->title == %s", searchTag->title);
				printf("\nsearchTag->description == %s", searchTag->desc);
			}
			
			mysql_escape_string(title, searchTag->title, strlen(searchTag->title));
			mysql_escape_string(desc, searchTag->desc, strlen(searchTag->desc));
			
			//			printf("\ntitle == %s", title);
			//			printf("\ndescription == %s", desc);
			
			
			//format as CSV and fill the buffer, broken into groups for readability
			/*			n += sprintf ((((struct mysqlTable*)object->myobject)->values[0] + n), "%d,%d,%d,\'\"%s\"\',\'\"%s\"\'\n",
			 searchTag->id,					//id is unique to this node
			 searchTag->recordID,		
			 searchTag->tableID,
			 searchTag->title,
			 searchTag->desc
			 );*/
			n += sprintf ((valsCopy + n), "%d,%d,%d,\'\"%s\"\',\'\"%s\"\'\n",
						  searchTag->id,					//id is unique to this node
						  searchTag->recordID,		
						  searchTag->tableID,
						  title,
						  desc
						  );
			
			
			
		}			
	}
	
	//	mysql_escape_string(escaped, buffer, strlen(buffer));
	if(debug == 1)
	{
		//printf("buffer : %s \nSize : %zu", buffer, strlen(buffer));
		//	pause();
	}
	
	if(numOfTags == 0)
	{
		printf("\nNo tags exist");
		return NULL;
	}
	else {
		//chunkValues(buffer, 65536);
		//chunkAllValues(object, buffer, dataRef);
		buffer = npNewChunkAllValues(valsCopy, numberOfChunks, dataRef);
	}
	
	
	//	for(int i = 0; i < *numberOfChunks; i++)
			printf("\nOUTPUTtext :: %s\n", buffer[0]);
	
	//after last group, end the node row using a newline
	//	n += sprintf ((buffer + n), "\n");	
	
	//increment the buffer
	data->io.write[kNPnode] += n;
	
	
	return buffer;
}

void npTraverseTreeDb (void (*nodeFunc)(pNPnode node, void* dataRef), 
					 pNPnode node, void* dataRef)
{
	int i = 0;

	nodeFunc(node, dataRef);

	for (i=0; i < node->childCount; i++)
	{
		// recursively call this function
		npTraverseTree (nodeFunc, node->child[i], dataRef);
		
		// apply passed in function operation to node
		//nodeFunc(node->child[i], dataRef); -- JJ, commented this out because it results in double calls, only need single call below
	}
	//process root node last -- JJ, moved from top so now root really is processed last
	

}

char*** npNewChunk(char **buffer, int arbritary, void *dataRef)
{
	char ***myBuffer = NULL;
	int i = 0, x = 0, y = 0;
	int z = 0;
	int num;
	int total = 0;
	
	while(buffer[i][0] != '\0')
	{
		printf("\n%d", i);
		total += strlen(buffer[i]);
		i++;
	}
	printf("\n%d", total);
	
	x = i;
//	num = (x / (total/arbritary)) + 220;
	num = (total/arbritary) + 120;
	myBuffer = malloc(sizeof(char*) * i);
//	for(i = 0; i < num; i++)
//		myBuffer[i] = malloc(sizeof(char*) * 350);
	
	total = 0;
	i = 0;
	
	while(buffer[i][0] != '\0')
	{
		total += strlen(buffer[i]);
		if(total <= arbritary)
		{
			myBuffer[z] = malloc(sizeof(char*) * 350);
			myBuffer[z][y] = buffer[i];
			i++;
			y++;
		}
		else if(total > arbritary)
		{
			y = 0;
			z++;
			total = 0;
		}
	}
	
	 
	return myBuffer;
}

char*** npChunk(char **buffer, int arbritary, void *dataRef)
{
	char ***myBuffer = NULL;
//	int numOfElements = 0;
	int total = 0;
//	int strTotal = 0;
	int test = 0;
	int i = 0;
	int x = 0;
	int y = 0;
	int z = 0;

	while(buffer[i][0] != '\0')
	{
		printf("\n%d", i);
		total += strlen(buffer[i]);
		i++;
	}
	printf("\n%d", total);
//	pause();
	
	x = i;
	myBuffer = malloc(sizeof(char*) * x);
	for(i = 0; i < x; i++)
		myBuffer[i] = malloc((sizeof(char*) * (total / arbritary)) + 1); // arbritary, debug db

//	myBuffer[i][0][0] = '\0';
	printf("\nx : %d", x);
	for(i = 0; i < x; i++)
	{
		test += strlen(buffer[i]);
		if(test <= arbritary) // This should be any arbritary number, debug db
		{
			myBuffer[y][z] = buffer[i];
			//printf("\nmyBuffer[%d][%d] : %s\n", y, z, myBuffer[y][z]);
			z++;
		}
		else
		{
			test = 0;
			myBuffer[y][z+1] = malloc(sizeof(char) * 2);
			myBuffer[y][z+1][0] = '\0';
			if(myBuffer[y][z] == '\0')
			{
				printf("\nFound Null 1");
			}

			if(myBuffer[y][z+1] == '\0') // N/A, debug db
			{
				printf("\nFound Null 2");
			}

//			printf("\nAdded null terminator : %s", myBuffer[y][z]);
//			printf("\nSecond null terminator : %s", myBuffer[y][z+1]);
		//	getch();
			z = 0;
			y++;
		}
	}
	printf("\nnumber of chunks : %d", y);
//	getch();
	/*
	if( (i == x) && (test <= arbritary))
	{
		printf("\nI is equal to X");
		
		myBuffer[y][z+1] = malloc(sizeof(char) * 2);
		myBuffer[y][z+1][0] = '\0';
		if(myBuffer[y][z] == '\0')
		{
			printf("\nFound Null 1");
		}

		getch();
	}
	*/

//	myBuffer[y][z] = malloc(sizeof(char) * 2); // ?, debug db
//	myBuffer[y][z][0] = '\0'; // ?, debug db

	myBuffer[y+1][0] = malloc(sizeof(char) * 2);
	myBuffer[y+1][0] = '\0';
//	printf("\nmyBuffer[%d][0][0]", i , myBuffer[i][0][0]);
//	getch();

	return myBuffer;
}

void npNewMapTraverseTreeLL(struct csvNodes *nodes, int* index, pNPnode node, int format, void* dataRef)
{
	pData data = (pData) dataRef;
	int i = 0;
//	int count = 0;
	
//	printf("\nIndex : %d", *index);
	//	printf("\nIndex is %d :: node->branchLevel : %d :: node->childCount : %d", *index, node->branchLevel, node->childCount);
	if (node->branchLevel > 0)
	{
		nodes->node[*index].size = npWriteNode(nodes->node[*index].csvStr, node, format, dataRef);
		nodes->totalCsvSize += nodes->node[*index].size;
		nodes->node[*index].csvStr[nodes->node[*index].size] = '\0';
	//	printf("\nCSV NODE %d : %s\nTotal Size : %d",*index, nodes->node[*index].csvStr, nodes->totalCsvSize);
		
		(*index)++;
		//data->io.write[kNPnode] = nodes->node[*index].csvStr; tmp
	}
	
	//recursively calls this function for all children
	for (i=0; i < node->childCount; i++)//node->childCount; j++)
	{
	//	printf("\nChild : %d", i);
		if (node->child[i] != NULL
			&& node->child[i]->child[0] != node)	//don't write twice, this is link B so skip
		{
			//count += npNewMapTraverseTreeLL(buffer, index, totalCount, arbritary, node->child[i], format, dataRef);
			//(*totalCount) = npNewMapTraverseTreeLL(buffer, index, totalCount, arbritary, node->child[i], format, dataRef);
			npNewMapTraverseTreeLL(nodes, index, node->child[i], kNPmapNodeCSV, dataRef);
			//data->io.write[kNPnode] = nodes->node[*index].csvStr; tmp
			//nodes->node[x].size = npMapTraverseTreeLL(nodes, &x, node, kNPmapNodeCSV, dataRef);
			//nodes->totalCsvSize += nodes->node[*index].size;
		}
	}
	//	printf("npMapTraverseTreeLL totalCount :: %d", count);
	
	
	//return count;
	//return nodes->totalCsvSize;
	
}




char **npGetNodesValues4(int arbritary, void *dataRef)
{
	pData data = (pData) dataRef;
	int i, x = 0;
	int count = 0;
	int totalCount = 0;
	pNPnode node = NULL;
	char **buffer = malloc(sizeof(char*) * data->map.nodeCount);
	
	for(i = 0; i < data->map.nodeCount; i++)
		buffer[i] = malloc(sizeof(char) * 3000);

	for(i = 0; i < data->map.nodeRootCount; i++) {
		node = data->map.node[i];
		data->io.write[kNPnode] = buffer[x];
		if(node->type != kNodeHUD)
		{
			count = npWriteNode(buffer[x], node, kNPmapNodeCSV, dataRef);
			totalCount += count;
			if(totalCount <= arbritary){
				buffer[x][count] = '\0';
				printf("\nBuffer[%d] : %s\n",x, buffer[x]);
				totalCount = 0;
			}
			else {
				buffer[x][count] = '\0';
				printf("\nBuffer[%d] : %s\n",x, buffer[x]);
			}
			x++;
			data->io.write[kNPnode] = buffer[x];
			if(node->childCount){
				//totalCount = npNewMapTraverseTreeLL(buffer, &x, &totalCount, arbritary, node, kNPmapNodeCSV, dataRef);
			}
			
		}
	}
	
	
}

char** npGetNodesValues3(int arbritary, void *dataRef)
{
	pData data = (pData) dataRef;
	int i, x = 0;
	int count = 0;
	int totalCount = 0;
	pNPnode node = NULL;
	char **buffer = malloc(sizeof(char*) * data->map.nodeCount);
	
	for(i = 0; i < data->map.nodeCount; i++)
		buffer[i] = malloc(sizeof(char) * 3000);
	
	for(i = 0; i < data->map.nodeRootCount; i++) {
		node = data->map.node[i];
		data->io.write[kNPnode] = buffer[x];
		if(node->type != kNodeHUD)
		{
			count = npWriteNode(buffer[x], node, kNPmapNodeCSV, dataRef);
			totalCount += count;
			if(totalCount >= arbritary){
				buffer[x][count] = '\0';
				buffer[x][count+1] = '\0';
				printf("\nBuffer[%d] : %s\n",x, buffer[x]);
				
				if( (buffer[x][count] == '\0') && (buffer[x][count+1] == '\0'))
				{
					printf("\nChunk");
				//	pause();
				}
					
				totalCount = 0;
			}
			else {
				buffer[x][count] = '\0';
				printf("\nBuffer[%d] : %s\n",x, buffer[x]);
			}
			x++;
			data->io.write[kNPnode] = buffer[x];
			if(node->childCount){
				//totalCount = npNewMapTraverseTreeLL(buffer, &x, &totalCount, arbritary, node, kNPmapNodeCSV, dataRef);
			}

		}
	}
	count = strlen(buffer[x]);
	buffer[x][count] = '\0';
	buffer[x][count+1] = '\0';
	buffer[x][count+2] = '\0';
	printf("\nBuffer[%d] : %s\n",x, buffer[x]);
	if( (buffer[x][count] == '\0') && (buffer[x][count+1] == '\0') && (buffer[x][count+2] == '\0'))
		printf("End OF Buffer");
	
	
	return buffer;
}


// pointify function?, debug db

struct csvNodes* npFixedNodeValues(void *dataRef)
{
	pData data = (pData) dataRef;
	pNPnode node = NULL;
	struct csvNodes *nodes;
	char buffer[64000];
	char **myBuffer;
	int totalCount = 0;
	//int count = 0;
	int i = 0;
	int x = 0;
	int y = 0;
	
	nodes = malloc(sizeof(nodes));
	nodes->numOfNodes = data->map.nodeCount;
	nodes->totalCsvSize = 0;
	nodes->node = malloc(sizeof(struct csvNode) * nodes->numOfNodes);
	printf("\nnumber of nodes : %d", nodes->numOfNodes);
//	getch();
	//pause();
	
	for(i = 0; i <= nodes->numOfNodes; i++)
		nodes->node[i].csvStr = malloc(sizeof(char) * 3000);
		
	for(i = 0; i < data->map.nodeRootCount; i++)
	{
		node = data->map.node[i];
	//	data->io.write[kNPnode] = nodes->node[x].csvStr;
		
		if (node->type != kNodeHUD)
		{
			nodes->node[x].size = npWriteNode (nodes->node[x].csvStr, node, kNPmapNodeCSV, dataRef);
			nodes->totalCsvSize += nodes->node[x].size;
			nodes->node[x].csvStr[nodes->node[x].size] = '\0';
			//printf("\nCSV NODE %d : %s",x, nodes->node[x].csvStr);
			
			x++;
		//	data->io.write[kNPnode] = nodes->node[x].csvStr;
			if (node->childCount)
			{
				npNewMapTraverseTreeLL(nodes, &x, node, kNPmapNodeCSV, dataRef);
			//	data->io.write[kNPnode] = nodes->node[x].csvStr;
				//nodes->node[x].size = npMapTraverseTreeLL(nodes, &x, node, kNPmapNodeCSV, dataRef);
			//	nodes->totalCsvSize += nodes->node[x].size; // debug db
				//totalCount += nodes->node[x].size;
				//totalCount += count;
			}
		}
		else {
			printf("\nnodes not being processed : %d", y+1);
	//		getch();
			y++;
		}


	}
	
	nodes->numOfNodes = x;
	printf("\nDONE : %d", x);
//	getch();
	return nodes;
}

// This should return a values structure containing a char** array of values and a int* array of lengths in the char** array, debug db
char** npGetNodesValues2(void *dataRef)
{
	pData data = (pData) dataRef;
	pNPnode node = NULL;
	int i = 0;
	int x = 0;
//	int y = 0;
//	int z = 0;
	int count = 0;
	int totalCount = 0;
//	int test = 0;
	char ***buffer = NULL;
	char **myBuffer = malloc(sizeof(char*) * data->map.nodeCount);

	printf("\nnpGetNodesValues2 : %i :: %i", data->map.nodeCount, data->map.nodeRootCount);

	for(i = 0; i < data->map.nodeCount; i++)
		myBuffer[i] = malloc(sizeof(char) * 3000);

	myBuffer[i+1] = malloc(sizeof(char) * 2);
	myBuffer[i+1][0] = '\0';

	for (i = 0; i < data->map.nodeRootCount; i++)
	{
		node = data->map.node[i];
		data->io.write[kNPnode] = myBuffer[x];
		
		if (node->type != kNodeHUD)
		{
			count = npWriteNode ((myBuffer[x]), node, kNPmapNodeCSV, dataRef);
			totalCount += count;
			myBuffer[x][count] = '\0';

			x++;
			data->io.write[kNPnode] = myBuffer[x];
			if (node->childCount)
			{
				count = npMapTraverseTreeLL(myBuffer, &x, node, kNPmapNodeCSV, dataRef);
				totalCount += count;
			}
		}
	}

	myBuffer[x][0] = '\0';

//	buffer = npChunk(myBuffer, 65536, dataRef);
//	printf("\nbuffer sizeof : %d", sizeof(buffer));
//	getch();

	printf("\n-----totalCount %d-----\n", totalCount);
//	getch();
	return myBuffer;
}

char** npGetNodesValues(char **buffer, int *numberOfChunks, void* dataRef)
{
	pData data = (pData) dataRef;
	pNPnode node = NULL;
	int count = 0;
	int secondCount = 0;
	int i = 0;

	//	int numberOfChunks;
	
//	char *valsCopy = malloc(10000 * data->map.nodeRootCount); // debug, doesn't account for child nodes
	char *valsCopy = malloc(3000 * data->map.nodeCount);

	if(data->map.nodeCount == 0)
	{
		return NULL;
	}
	
	//char *valsCopy = malloc(3000 * data->map.nodeCount);
	
	
	printf("\nnpRetrieveNodeValues1 : We have this many nodes : %d", data->map.nodeRootCount);
	
	data->io.write[kNPnode] = valsCopy;
	
	count = 0;
	
	//This portion should be in its own function
	//This for loop could act as the chunker, debug db
	/*
	for (i=0; i < data->map.nodeRootCount; i++)
	{
		node = data->map.node[i];

		if (node->type != kNodeHUD)				// Do something about that not statement later, debug ll						
		{
			count += npWriteNode ((valsCopy + count), node, kNPmapNodeCSV, dataRef);
			secondCount += count;

			if (node->childCount)
			{
				count += npMapTraverseTree (valsCopy, node, kNPmapNodeCSV, dataRef);
				secondCount += count;
			}
		}
	}
	printf("\nCount : %d", count);
	*/

	for (i=0; i < data->map.nodeRootCount; i++)
	{
		node = data->map.node[i];

		if(secondCount >= 65536)
		{
			printf("\nsecondCount : %d", secondCount);
			secondCount = 0;
		//	getch();
		}
		else
		{
			printf("\ncount : %d", count);
		}

		if (node->type != kNodeHUD)				// Do something about that not statement later, debug ll						
		{
			count = npWriteNode ((valsCopy), node, kNPmapNodeCSV, dataRef);
			secondCount += count;

			if (node->childCount)
			{
				count = npMapTraverseTree (valsCopy, node, kNPmapNodeCSV, dataRef);
				secondCount += count;
			}
		}
	}
	printf("\nCount : %d", count);


	buffer = npNewChunkAllValues(valsCopy, numberOfChunks, dataRef);
	
	
//	for(int i = 0; i < *numberOfChunks; i++)
//		printf("\nOUTPUT2 :: %s\n", buffer[i]);
	
	//	pause();
	return buffer;
}

int npFormatChunk(char *buffer, char* formattedValues, char* values)
{
	//Free after use
	int count = 0;
	int curs;
	int totSize = strlen(buffer);
	int i = 0;
	memset(values, '\0', 300000);
	memset(formattedValues, '\0', 300000);
	
	printf("\nSize of buffer : %d\n", totSize);
	while(count < totSize)
	{
		count += curs = npSeekToNextLine(buffer);
		
		if(count > totSize) //npSeekToNextLine doesn't check if it has reached end of buffer so this error checking is necessary, debug ll
		{
			printf("\nCOUNT EXCEEDED TOTSIZE!!!!\n");
			break;
		}
			
	//	printf("\ncurs : %d\n", curs);
		for(i = 0; i < curs; i++)
		{
			values[i] = buffer[i];
		}
		values[i+1] = '\0';
		// If not empty
		if(curs > 1) // ??? curs < 500
		{
		strcat(formattedValues, "(");
		strcat(formattedValues, values);
		strcat(formattedValues, ")");
		strcat(formattedValues, ",");
		}

		memset(values, '\0', curs+1);
		
		buffer = &buffer[curs];
	}
	
	formattedValues[strlen(formattedValues)-1] = '\0';
	//printf("\nnpFormatChunk : %s\n", formattedValues);
	
	//	pause();
	return strlen(formattedValues);
}

// All 2-3 dimensional arrays should be terminated by a null character, this numOfChunks variable is messy business, debug db
char** npNewChunkAllValues(char* buffer, int *numOfChunks, void* dataRef)
{
//	pData data = (pData) dataRef;
	double tester;
	float holder;
	float mod;
	int i;
	char **chunked;
	//	int numOfChunks = 0;
	
	int count = 0;
	printf("\nnpNewChunkAllValues");
//	getch();
	//	printf("npFillerFunc : %d", strlen(valsCopy));
	if ( (sizeof(char) * strlen(buffer)) > 1048576)
	{
		//		printf("\nThis is going to be too large : vals[0] : \n%s\n", table->values[0]); // I might have to restructure the data structure
		*numOfChunks =  ((strlen(buffer) * sizeof(char))/(float)65536); // This shouldn't be here, debug ll
		holder = (strlen(buffer)*sizeof(char))/(float)65536;
		mod = modf( holder , &tester);
		
		if(mod > 0)
			(*numOfChunks)++;
		
		printf("NUMBER OF CHUNKS : %d", *numOfChunks);
		
		
		chunked = malloc(sizeof(char*) * *numOfChunks);
		for(i = 0; i < *numOfChunks; i++)
			chunked[i] = malloc(80000 * sizeof(char));
		
		//		((struct mysqlTable*)object->myobject)->values = malloc(sizeof(char*) * numOfChunks);
		//		*(((struct mysqlTable*)object->myobject)->values) = malloc(sizeof(char) * 80000);
		
		for(i = 0 ; i < *numOfChunks; i++)
		{
			count = fixedChunkValues(buffer, chunked[i], 65536);
			//		((struct mysqlTable*)object->myobject)->values[i] = chunked[i];
			buffer = &buffer[count];
			//		printf("\nOUTPUT :: %s\n", chunked[i]);
		}
		
		printf("\nReturned from chunkValues()");
	}
	else {
		*numOfChunks =  1; // This maybe shouldn't be here, debug ll
		//		((struct mysqlTable*)object->myobject)->values = malloc(sizeof(char*) * ((struct mysqlTable*)object->myobject)->numOfChunks);
		//		*(((struct mysqlTable*)object->myobject)->values) = malloc(sizeof(char) * 80000); // Make this dynamic
		//		((struct mysqlTable*)object->myobject)->values[0] = valsCopy;
		chunked = malloc(sizeof(char*) * *numOfChunks);
		chunked[0] = malloc(80000 * sizeof(char));
		chunked[0] = buffer;
		return chunked;
	}
	
	return chunked;
}

int fixedChunkValues(char* buffer, char* chunked, int chunkSize)
{
	int count = 0, curs = 0;
	
	if (strlen(buffer) < chunkSize)
		count = strlen(buffer);
	else 
	{
		
	while(count <= chunkSize)
		count += curs = npSeekToNextLine(&buffer[curs]);
	
		while(buffer[count++] != '\n');
		
	}
			
	memcpy(chunked, buffer, count); // This might be count + curs;
	chunked[count] = '\0';

	return count;
}
